package clews;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Stack;

import javax.swing.JOptionPane;
import javax.swing.JTextArea;

import clews.History.Operation;
import clews.data.Configuration;
import clews.data.Constraint;
import clews.data.DataObject;
import clews.data.Specification;
import clews.gui.DomainTree;
import clews.gui.DomainTreeModel;
import clews.gui.diagram.ConfigurationDiagram;
import clews.gui.diagram.DiagramPanel;
import clews.gui.diagram.SpecificationDiagram;
import clews.gui.view.ClassView;
import clews.gui.view.ConfigurationView;
import clews.gui.view.ConstraintView;
import clews.gui.view.SpecificationView;

public class Domain {
	
	protected Specification specification;
	protected ArrayList<Configuration> configurations;
	protected MainFrame mainFrame;
	protected DomainTree domainTree;
	protected JTextArea textOutput; 
	protected DiagramPanel diagramPanel;
	protected String path;
	protected History history;

	public Domain()
	{
		this(null);
		init();
	}
	
	public Domain(MainFrame mf)
	{
		mainFrame = mf;
		if (mainFrame != null){
			history = new History(mainFrame);
			history.clearUndoRedo();
		}
	}
	
	
	public void init()
	{
		path = null;
		if (mainFrame != null){
			domainTree = mainFrame.getDomainTree();
			textOutput = mainFrame.getTextOutput();
			diagramPanel = mainFrame.getDiagramPanel();
		}
		newDomain();
	}
	
	public void setPath(String p)
	{
		path = p;
	}
	
	public String getPath()
	{
		return path;
	}
	
	public Specification getSpecification()
	{
		return specification;
	}
	
	public ArrayList<Configuration> getConfigurations()
	{
		return configurations;
	}
	
	public void setConfigurations(ArrayList<Configuration> configs)
	{
		configurations = configs;
	}
	
	public History getHistory()
	{
		return history;
	}

	public Configuration addNewConfiguration()
	{
		if (history != null)
			history.addUndo(null,Operation.ADD,"New Configuration added");
		Configuration c =  addNewConfiguration(null);
		return c;
	}
	
	public Constraint addNewConstraint()
	{
		if (history != null)
			history.addUndo(null,Operation.ADD,"New Constraint added");
		Constraint c = specification.addNewConstraint();
		ConstraintView cv = new ConstraintView(c);
		c.setView(cv);
		if (domainTree != null)
			domainTree.updateUI();
		cv.edit();
		return c;
	}
	
	protected Configuration addNewConfiguration(Configuration exc)
	{
		Configuration c = exc==null?new Configuration(specification):exc;
		ConfigurationView cv = new ConfigurationView(c);
		c.setView(cv);
		configurations.add(c);
		if (mainFrame != null){
			mainFrame.getDiagramPanel().changeSelectedDiagram(cv.getDiagram());
			mainFrame.update();
		}
		return cv.getConfiguration();
	}	

	
	public void addNewMinimalConfiguration()
	{
		if (specification.isConsistent())
			specification.createMinimalConfiguration();
		else{
			if (mainFrame != null)
				JOptionPane.showConfirmDialog(mainFrame, "Cannot create configuration since the specification is not consistent","Creation not possible...",JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	public clews.data.Class addNewClass()
	{
		if (history != null)
			history.addUndo(null,Operation.ADD,"New Class added");
		clews.data.Class c = specification.addNewClass();
		if (mainFrame != null){
			ClassView cv = specification.getView().addNewClassView(c);		
			mainFrame.getDomainTree().updateUI();
		}
		return c;
	}
	
	public void newDomain()
	{
		newDomain(true);
	}
	
	public void newDomain(boolean reset)
	{
		if (reset)
			DataObject.resetIds();
		specification = new Specification();
		configurations = new ArrayList<Configuration>();
		if (mainFrame != null){ 
			SpecificationView sv = new SpecificationView(specification);
			specification.setView(sv);		
			
			if (mainFrame.getDiagramPanel() != null)
				mainFrame.getDiagramPanel().resetView();
		}
		setSpecification(specification);
	}
	
	public void setActiveDiagram(String active)
	{
		if (diagramPanel != null){
			if (specification.getView().getDiagram().getName().equals(active))
				diagramPanel.changeSelectedDiagram(specification.getView().getDiagram());
			for (Configuration c : configurations)
				if (c.getView().getDiagram().getName().equals(active))
					diagramPanel.changeSelectedDiagram(c.getView().getDiagram());
		}
	}
	
	public void classDroped(clews.data.Class c, Point p)
	{
		if (diagramPanel != null){
			Point tp = new Point();
			diagramPanel.getDiagram().getInverseTransform().transform(p, tp);
			if (diagramPanel.getDiagram() instanceof ConfigurationDiagram && c != null){
				history.addUndo(null,Operation.ADD,"New Instance added");
				((ConfigurationDiagram)diagramPanel.getDiagram()).getConfigurationView().addDropInstance(c,tp);
			}
			if (diagramPanel.getDiagram() instanceof SpecificationDiagram)
				addNewClass().getView().setPosFromPosition(tp);			
			diagramPanel.repaint();
		}
	}
	
	public void setSpecification(Specification spec)
	{		
		specification = spec;
		for (Constraint c: specification.getConstraints())
			c.computeResultingInterval();
		
		if (diagramPanel != null)
			diagramPanel.changeSelectedDiagram(spec.getView().getDiagram());

		if (mainFrame != null){
			System.out.println(domainTree);
			domainTree.setModel(new DomainTreeModel(this));
			for (int i=0;i<domainTree.getRowCount();i++)
				domainTree.expandRow(i);
			mainFrame.repaint();
		}
	}

	public void extendConfiguration()
	{
		if (diagramPanel != null){
			if (diagramPanel.getDiagram() instanceof ConfigurationDiagram)
			{
				Configuration c = ((ConfigurationDiagram)diagramPanel.getDiagram()).getConfigurationView().getConfiguration();
				c.extendToInstance();
			}
		}
	}
	
}
