package clews;

import java.io.File;
import java.util.Stack;

import clews.env.Environment;
import clews.export.DomainXMLExport;
import clews.export.DomainXMLImport;
import clews.gui.view.ClassView;


public class History {
	protected Stack<HistoryObject> undoStack;
	protected Stack<HistoryObject> redoStack;
	protected MainFrame mainFrame;
	
	public History(MainFrame mf)
	{
		mainFrame = mf;
	}
	
	public void doUndo()
	{
		try{
			HistoryObject ho = undoStack.peek();
			
			File f = File.createTempFile("redo","."+Environment.DOMAIN_ENDING,new File(Environment.HISTORY_PATH));
			f.deleteOnExit();			
			DomainXMLExport dxe = new DomainXMLExport(MainFrame.getInstance().getDomain());
			dxe.saveToDisk(f.getAbsolutePath(),true);

			switch (ho.operation)
			{
			case ADD:
				if (ho.object instanceof ClassView)
					undoAddClass((ClassView)ho.object);			
				break;
			case CHANGE:
				break;
			case DELETE:
				if (ho.object instanceof ClassView)
					undoDeleteClass((ClassView)ho.object);
				break;
			}
			DomainXMLImport dxi = new DomainXMLImport();
			dxi.loadFromDisk(ho.file.getAbsolutePath());
			
			redoStack.push(undoStack.pop());
			redoStack.peek().file = f;
			mainFrame.getMainMenu().refreshUnRedo();
			
			Domain d = mainFrame.getDomain();
			String old = d.getPath();
			d.newDomain(false);
			d.setPath(old);
			d.setConfigurations(dxi.getConfigurations());
			d.setSpecification(dxi.getSpecification());	    
			d.setActiveDiagram(dxi.getActive());
		}catch (Exception e)
		{			
			e.printStackTrace();
		}
	}
	
	public void undoAddClass(ClassView cv)
	{
		
	}
	
	public void undoDeleteClass(ClassView cv)
	{
		
	}
	
	public void doRedo()
	{
		try{
			HistoryObject ho = redoStack.peek();
			undoStack.push(redoStack.pop());
			mainFrame.getMainMenu().refreshUnRedo();
			
			File f = File.createTempFile("undo","."+Environment.DOMAIN_ENDING,new File(Environment.HISTORY_PATH));
			f.deleteOnExit();			
			DomainXMLExport dxe = new DomainXMLExport(MainFrame.getInstance().getDomain());
			dxe.saveToDisk(f.getAbsolutePath(),true);

			DomainXMLImport dxi = new DomainXMLImport();
			dxi.loadFromDisk(ho.file.getAbsolutePath());
			undoStack.peek().file = f;
			
			Domain d = mainFrame.getDomain();
			String old = d.getPath();
			d.newDomain(false);
			d.setPath(old);
			d.setConfigurations(dxi.getConfigurations());
			d.setSpecification(dxi.getSpecification());
			d.setActiveDiagram(dxi.getActive());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public Stack<HistoryObject> getUndoStack()
	{
		return undoStack;
	}

	public Stack<HistoryObject> getRedoStack()
	{
		return redoStack;
	}
	
	public void clearUndoRedo()
	{
		undoStack = new Stack<HistoryObject>();
		redoStack = new Stack<HistoryObject>();	
		if (mainFrame.getMainMenu() != null)
			mainFrame.getMainMenu().refreshUnRedo();
	}
	
	public void addUndo(Object obj,Operation op, String msg)
	{
		try{
			File f = File.createTempFile("undo","."+Environment.DOMAIN_ENDING,new File(Environment.HISTORY_PATH));
			f.deleteOnExit();
			
			undoStack.push(new HistoryObject(obj, op, msg,f));
			redoStack = new Stack<HistoryObject>();
			mainFrame.setMessageText(msg+"...");
			mainFrame.getMainMenu().refreshUnRedo();
			
			DomainXMLExport dxe = new DomainXMLExport(MainFrame.getInstance().getDomain());
			dxe.saveToDisk(f.getAbsolutePath(),true);
		}catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public enum Operation {
		ADD,
		DELETE,
		CHANGE
	}
	
	public class HistoryObject {
		
		protected Object object;
		protected Operation operation;	
		protected String message;
		protected File file;
		
		public HistoryObject(Object obj, Operation op, String msg, File f)
		{
			object = obj;
			operation = op;
			message = msg;
			file = f;
			if (!message.startsWith("'"))
				message = "'"+message;
			if (!message.endsWith("'"))
				message = message+"'";
		}
		
		public String toString()
		{
			return message;
		}
	}	
}
