package clews;


import java.io.File;

import javax.swing.JOptionPane;

import javax.swing.UIManager;

import net.sf.javailp.Linear;
import net.sf.javailp.Problem;
import net.sf.javailp.Solver;
import net.sf.javailp.SolverFactory;
import net.sf.javailp.SolverFactoryLpSolve;

import clews.env.Environment;

public class Main {

	public static void doInit()
	{
		Environment.getStandardText("init");
		if (!MainFrame.clearHistory()){
				System.out.println("Error during startup. Cannot create temporary directory");
		}
	}
	
	public static void main(String[] args)
	{

		doInit();
//		setNativeLookAndFeel();
		
		MainFrame cmf = MainFrame.getInstance();
		checkILPSolver();
		cmf.startUp();
	}
	
	public static void checkILPSolver()
	{
		try
		{
			SolverFactory factory = new SolverFactoryLpSolve(); // use lp_solve
			factory.setParameter(Solver.VERBOSE, 0);
			factory.setParameter(Solver.TIMEOUT, 100); // set timeout to 100 seconds
			Solver solver = factory.get(); 
			Problem p = new Problem();
			Linear l = new Linear();			
			l.add(1, "x"); 
			p.add(l, "<=", 12);			
			solver.solve(p);
		}catch (Exception e)
		{
			JOptionPane.showMessageDialog(MainFrame.getInstance(), 					
					"No ILP-Solver found! Clews will not work properly.");
		}
	}

	public static void setNativeLookAndFeel() {
		
	    try {
	      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	    } catch(Exception e) {
	      System.out.println("Error setting native LAF: " + e);
	    }
	  }

}
