package clews;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import clews.env.Environment;
import clews.gui.DomainTree;
import clews.gui.MainKeyListener;
import clews.gui.MainMenu;
import clews.gui.diagram.DiagramPanel;

public class MainFrame 
extends JFrame
{
	private static final long serialVersionUID = -4520788081029975623L;
	protected static MainFrame instance = null;
	protected DomainTree domainTree;
	protected JTextArea textOutput; 
	protected DiagramPanel diagramPanel;
	protected JLabel messageText;
	protected MainMenu mainMenu;
	protected Domain domain;
	
	protected MainFrame()
	{
	    super("clews");
	}
/*	
	@Override
	public void paint(java.awt.Graphics g) 
	{
		System.out.println("Paint!");
		super.paint(g);
	}
	
	@Override
	public void paintAll(java.awt.Graphics g) 
	{
		System.out.println("PaintAll!");
	}
	
	@Override
	public void paintComponents(java.awt.Graphics g) 
	{
		System.out.println("PaintComponents!");
	}
*/	
	public static MainFrame getInstance()
	{
		if (instance == null)
			instance = new MainFrame();
		return instance;
	}
	
	public DomainTree getDomainTree()
	{
		return domainTree;
	}
	
	public JTextArea getTextOutput()
	{
		return textOutput;
	}
	
	public DiagramPanel getDiagramPanel()
	{
		return diagramPanel;
	}
	
	public MainMenu getMainMenu()
	{
		return mainMenu;
	}
	
	public Domain getDomain()
	{
		return domain;
	}
	
	public void update()
	{
		setMessageText(" ");
		domainTree.updateUI();
		diagramPanel.repaint();
	}
	
	public void setMessageText(String text)
	{
		messageText.setText(text);
//		messageText.sets
		messageText.updateUI();
	}
	
	public void startUp()
	{
		addListener();
		setUpLayout();
	    domain = new Domain(this);
		domain.init();
	}
	
	protected void addListener()
	{
	    addKeyListener(new MainKeyListener(this));
	    addWindowListener(new WindowAdapter() {
	        public void windowClosing(WindowEvent e) 
	        {
	        	MainFrame.clearHistory();
	        	System.exit(0);
	        }
	    });		
	}
	
	public static boolean clearHistory()
	{
		File f = new File(Environment.HISTORY_PATH);
		if (!f.exists()){
			if (!f.mkdir())
				return false;
		}
		File[] files = f.listFiles();
		for (int i=0;i<files.length;i++)
			files[i].delete();
			
		return true;
	}
	
	protected void setUpLayout()
	{
		setUpLayout(true);
	}
	
	protected void setUpLayout(boolean show)
	{
	    if (show){
			setLocation(32, 32);
		    setSize(new Dimension(1024,768));	    
		    setExtendedState(getExtendedState() | JFrame.MAXIMIZED_BOTH);
	//	    pack();
		    setVisible(true);
		    setVisible(false);
	    }
	    //setLayout(new BorderLayout());
	
	    JPanel contentPane = new JPanel(new BorderLayout());
	    JPanel left = new JPanel();
	    left.setLayout(new BorderLayout());
	    domainTree = new DomainTree(this);
	    left.add(domainTree);
	
	    
	    diagramPanel = new DiagramPanel(this);
		
	    JPanel lower = new JPanel();		
		JSplitPane right = new JSplitPane(
		      JSplitPane.VERTICAL_SPLIT, diagramPanel, lower);		
		JSplitPane pane = new JSplitPane(
		  	      JSplitPane.HORIZONTAL_SPLIT, left, right);
		
	    lower.setLayout(new BorderLayout());
	    JTextArea ta = new JTextArea();
	    ta.setEditable(false);
	    JScrollPane scroll = new JScrollPane(ta);
		    
		lower.add(scroll);
		textOutput = ta;	    
	
		contentPane.add(pane);		
		
		messageText = new JLabel(" "); // otherwise label is not visible after starting
		contentPane.add(messageText, BorderLayout.SOUTH);
			    
		setJMenuBar(mainMenu=new MainMenu(this));

		contentPane.setOpaque(true);
	    setContentPane(contentPane);
//	    setLocation(32, 32);
//	    setSize(new Dimension(1024,768));	    
	    setExtendedState(getExtendedState() | JFrame.MAXIMIZED_BOTH);
//	    pack();
	    if (show)
			setVisible(true);	    

		right.setDividerLocation(0.85);
		pane.setDividerLocation(0.15);
		
		pane.repaint();	
		domainTree.updateUI();
		mainMenu.updateUI();
		messageText.repaint();
		update();
	}
}
