package clews.data;

import java.util.ArrayList;

import clews.data.Constraint.ConstraintPart;
import clews.env.Environment;
import clews.gui.view.AssociationView;

public class Association extends DataObject<AssociationView>{

	protected ArrayList<AssociationEnd> ends;
	protected ArrayList<ConstraintPart> constraints;
	protected AssociationEnd left,right;
	protected boolean isConsistent;
	protected String name;
	
	public Association(Class c1, Class c2)
	{
		isConsistent = true;
		constraints = new ArrayList<ConstraintPart>();
		ends = new ArrayList<AssociationEnd>();
		ends.add((left=new AssociationEnd(this,AssociationAttribute.UNIQUE, 0, Environment.CARDINALITY_STAR, c1)));
		ends.add((right=new AssociationEnd(this,AssociationAttribute.UNIQUE, 0, Environment.CARDINALITY_STAR, c2)));
		left.setOther(right);
		right.setOther(left);
		name = left.assclass.toString().toLowerCase() + "TO" + right.assclass.toString().toLowerCase();
		name = name.replace(" ", "");
	}
	
	public Association(ArrayList<AssociationEnd> ae)
	{
		ends = ae;
		constraints = new ArrayList<ConstraintPart>();
		left = ae.get(0);
		right = ae.get(1);
		left.setOther(right);
		right.setOther(left);
		for (AssociationEnd e : ends)
			e.assoc = this;
	}
	
	public Association(Class c1, Class c2, AssociationView av)
	{
		this(c1,c2);
		setView(av);
	}
	
	public ArrayList<ConstraintPart> getConstraints()
	{
		return constraints;
	}
	
	public void deleteConstraint(ConstraintPart c)
	{
		constraints.remove(c);	
	}
	
	public void addConstraint(ConstraintPart c)
	{
		constraints.add(c);
	}
	
	public String getName()
	{
		return name;
	}

	public void setName(String n)
	{
		name = n;
	}
	
	public boolean isBinary()
	{
		return (getArity()==2);
	}
	
	public boolean isConsistent()
	{
		return isConsistent;
	}
	
	public void setIsConsistent(boolean val)
	{
		isConsistent = val;
	}
	
	public int getArity()
	{
		return ends==null?0:ends.size();
	}
	
	public ArrayList<AssociationEnd> getEnds()
	{
		return ends;
	}
	
	public AssociationEnd getLeft()
	{
		return left;
	}
	
	public AssociationEnd getRight()
	{
		return right;
	}
	
	
	public clews.data.Class getLeftClass()
	{
		return left.assclass;
	}
	
	public clews.data.Class getRightClass()
	{
		return right.assclass;
	}
	
	public AssociationAttribute getTypeLeft() {
		return left.type;
	}
	
	public AssociationAttribute getTypeRight() {
		return right.type;
	}
	
	public int getMinLeft() {
		return left.getMin();
	}
	
	public int getMaxLeft() {
		return left.getMax();
	}
	
	public int getMinRight() {
		return right.getMin();
	}
	
	public int getMaxRight() {
		return right.getMax();
	}
	
	public int getConstrainedMaxLeft() {
		if (left.type != AssociationAttribute.UNIQUE)
			return getMaxLeft();
		int ret = getMaxLeft();
		if (ret == Environment.CARDINALITY_STAR)
			ret = Integer.MAX_VALUE;
		for (ConstraintPart cp : constraints)
			if (cp.constraint.isActive())
			{
				int cur=0;
				if (cp.inv)
				{
					if (right.getMax()==20){
						ret = 12;
						break;
					}
					if (left.getMin() != 0)
						cur = cp.constraint.getLowerProduct(cp) / left.getMin();
					if (cur > 0){
						cur = (cp.constraint.getUpperBound()+cur-1) / cur; 
						ret = Math.min(ret, cur);
					}else
						ret = Math.min(ret, cp.constraint.getUpperBound());
				}
			}
		if (ret == Integer.MAX_VALUE)
			ret = Environment.CARDINALITY_STAR;
		return ret;
	}
	
	public int getConstrainedMaxRight() {
		if (right.type != AssociationAttribute.UNIQUE)
			return getMaxRight();
		int ret = getMaxRight();
		if (ret == Environment.CARDINALITY_STAR)
			ret = Integer.MAX_VALUE;
		for (ConstraintPart cp : constraints)
			if (cp.constraint.isActive())
			{
				int cur=0;
				if (!cp.inv)
				{
					if (right.getMax()==20){
						ret = 12;
						break;
					}
					if (right.getMin() != 0)
						cur = cp.constraint.getLowerProduct(cp) / right.getMin();
					if (cur > 0){
						cur = (cp.constraint.getUpperBound()+cur-1) / cur; 
						ret = Math.min(ret, cur);
					}else
						ret = Math.min(ret, cp.constraint.getUpperBound());						
				}
			}
		if (ret == Integer.MAX_VALUE)
			ret = Environment.CARDINALITY_STAR;
		return ret;
	}
	
	public void setCards(int mil, int mal, int mir, int mar)
	{
		left.setMinMax(mil,mal);
		right.setMinMax(mir,mar);
	}
	
	public void setTypes(AssociationAttribute l, AssociationAttribute r)
	{
		left.type = l;
		right.type = r;
	}
	
	public String toString()
	{
		return getName();
	}
}
