package clews.data;

import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;

import net.sf.javailp.Result;

import clews.MainFrame;
import clews.env.Graph;
import clews.env.Utils;
import clews.export.SpecificationILPExport;
import clews.gui.diagram.DiagramPanel;
import clews.gui.view.ConfigurationView;

public class Configuration
extends DataObject<ConfigurationView>
{
	protected String name;
	protected Specification specification;
	protected ArrayList<Instance> instances;
	protected ArrayList<Link> links;
	protected String path;

	
	public Configuration(Specification s)
	{
		specification = s;
		setName("Configuration"+this.id);
		instances = new ArrayList<Instance>();
		links = new ArrayList<Link>();
	}
	
	public String getPath()
	{
		return path;
	}
	
	public void setPath(String p)
	{
		path = p;
	}
	
	public void delete(Instance in)
	{
		instances.remove(in);		
	}
	
	public void delete(Link li)
	{
		li.left.removeLink(li,li.right,(li.getView()!=null)?li.getView().getRightCnt():0);
		li.right.removeLink(li,li.left,(li.getView()!=null)?li.getView().getLeftCnt():0);
		links.remove(li);
		applyChanges();
	}
	
	public void clear()
	{
		instances.clear();
		links.clear();
	}
	
	public void applyChanges()
	{
		for (Instance i : instances)
		{
			i.applyChanges();
		}
	}
	
	public Link link(ArrayList<Instance> ends)
	{
		if (ends.size() == 2)
			return link(ends.get(0),ends.get(1));
		else
		{
			// TODO 
		}
		return null;
	}
	
	public Link link(Instance i1, Instance i2)
	{
		Link l = new Link(this,i1,i2);
		i1.addLink(l,i2);
		i2.addLink(l,i1);
		links.add(l);
		i1.applyChanges();
		i2.applyChanges();
		return l;
	}
	
	public void addMinLinks(AssociationEnd a1, AssociationEnd a2)
	{
		ArrayList<Instance> i1 = new ArrayList<Instance>(), i2= new ArrayList<Instance>();
		clews.data.Class c1 = a1.assclass, c2 = a2.assclass;
		
		for (Instance i: instances)
		{
			if (i.getType().getId() == c1.getId())
				i1.add(i);
			if (i.getType().getId() == c2.getId())
				i2.add(i);
		}
		
		int cnt1=i1.size(),cnt2=i2.size();
		
		if (cnt1 > 0 && cnt2 > 0)
		{
			int ii1=0,ii2=0,lcm=Utils.lcm(cnt1,cnt2);
						
			int l = getNumberOfLinks(cnt1,a1,cnt2,a2);
			
	//		System.out.println("Links: "+l);
			for (int i=0;i<l;i++)
			{			
				ii1 = i % cnt1;
				ii2 = (i + (i/lcm)) % cnt2;
	//			System.out.println("link> "+ii1+" "+ii2);
				Link tl = link(i1.get(ii1), i2.get(ii2));
				if (view != null)
					view.addNewLinkView(tl);
			}
		}
	}	
	
	protected int getNumberOfLinks(int c1, AssociationEnd a1, int c2, AssociationEnd a2)
	{
		if (a2.type == AssociationAttribute.UNIQUE || 
		   (a2.type == AssociationAttribute.NON_UNIQUE && a1.type == AssociationAttribute.ALL_SAME))
		{
			AssociationEnd tmpa = a1;
			a1 = a2;
			a2 = tmpa;
			int tmpc = c1;
			c1 = c2;
			c2 = tmpc;
		}
		
		if (a1.type == a2.type)
		{
			switch (a1.type)
			{
			case UNIQUE: return Math.max(c1*a2.getMin(), c2*a1.getMin());
			case NON_UNIQUE: return Math.max(c1*a2.getMin(), c2*a1.getMin());
			case ALL_SAME: return Math.min(c1,c2) * Math.max(a1.getMin(), a2.getMin());
			}
		}
		else
		if (a1.type == AssociationAttribute.UNIQUE && a2.type == AssociationAttribute.NON_UNIQUE)
			return Math.max(c1*a2.getMin(), c2*a1.getMin());
		else if (a1.type == AssociationAttribute.UNIQUE && a2.type == AssociationAttribute.ALL_SAME)
			return Math.max(c1*a2.getMin(), c2*a1.getMin());
		else if (a1.type == AssociationAttribute.NON_UNIQUE && a2.type == AssociationAttribute.ALL_SAME)
			return Math.max(c1*a2.getMin(), c2*a1.getMin());
		
		return -1;
	}
	
	public Instance getInstanceById(int id)
	{
		for (Instance i: instances)
			if (i.getId() == id)
				return i;				
		return null;
	}
	
	public Instance addInstance(clews.data.Class c)
	{
		int cnt = 1;
		for (Instance i: instances)
			if (i.getType().getId() == c.getId())
				cnt++;
			
		Instance inst = new Instance(this,c);		
		inst.setName(c.getName()+"("+cnt+")");
		instances.add(inst);
		inst.applyChanges();
		return inst;
	}
	
	public ArrayList<Instance> getInstances()
	{
		return instances;
	}
	
	public ArrayList<Link> getLinks()
	{
		return links;
	}
	
	public void setName(String n)
	{
		name = n;
	}
	
	public Specification getSpecification()
	{
		return specification;
	}

	public String getName()
	{
		return name;
	}
	
	public String toString()
	{
		return getName();
	}
	
	public void extendToInstance(){		
		HashMap<clews.data.Class, Integer> minObjects;
		minObjects = new HashMap<Class, Integer>();
		
		for (clews.data.Class c : getSpecification().getClasses())
			minObjects.put(c, 0);
		
		for (Instance i : instances)
			minObjects.put(i.getType(), minObjects.get(i.getType()).intValue()+1);
		
		SpecificationILPExport ilp = new SpecificationILPExport(getSpecification(),minObjects);
		ilp.export();
		if (ilp.solve() != null)
		{
			Dimension d = MainFrame.getInstance().getDiagramPanel().getSize();
			for (clews.data.Class c : getSpecification().getClasses()){
				int v = ilp.getResultVal(c)-minObjects.get(c).intValue();
				for (int k=0;k<v;k++)
					getView().addDropInstance(c,new Point((int)(Math.random()*d.width),(int)(Math.random()*d.height)));
				minObjects.put(c, minObjects.get(c).intValue()+v);
			}
		}	
		
		for (Association a : specification.getAssociations()){
			ArrayList<Instance> l = getInstancesForType(a.getLeftClass());
			ArrayList<Instance> r = getInstancesForType(a.getRightClass());
			ArrayList<Link> ex = new ArrayList<Link>();
			for (Link ll : links){
				if ((l.contains(ll.left) && r.contains(ll.right)) ||
					(r.contains(ll.left) && l.contains(ll.right)))
					ex.add(ll);
			}
			
			Graph g = new Graph();
			g.init(getNumberOfLinks(l.size(),a.getLeft(),r.size(),a.getRight()), l,r,ex);
			if (g.findFeasible()){
				for (Instance cl : l)
					for (Instance cr : r)
						if (g.isFlow(cl,cr)){
							getView().addNewLinkView(link(cl,cr));
						}
			}
		}
	}
	
	protected ArrayList<Instance> getInstancesForType(clews.data.Class c){
		ArrayList<Instance> ret = new ArrayList<Instance>();
		for (Instance i : instances)
			if (i.type == c)
				ret.add(i);
		return ret;
	}
	
	public boolean[][] findLinks(int l, int n, int m, boolean[][] fixed)
	{
		boolean[][] res = new boolean[n][m];
		int i,j,k,minl,minr,maxl,maxr;
		int[] fleft = new int[n];
		int[] fright = new int[m];
		/*
		for (i=0;i<l;i++){			
			for (j=0;j<n;j++)
				for (k=0;k<m;k++)
					if (!res[j][k] && fleft[j] < maxl && fright[k] < maxr){
						res[j][k] = true;
						fleft[j]++;
						fright[k]++;
					}
						
		}
		*/
		return res;
	}
	
}
