package clews.data;

import java.util.ArrayList;
import java.util.HashMap;
import clews.env.Environment;
import clews.env.Utils;
import clews.gui.view.InstanceView;

public class Instance 
extends DataObject<InstanceView>
{
	protected clews.data.Class type;
	protected String name;
	protected Configuration config;
	protected HashMap<Number,ArrayList<Link>> links;
	protected HashMap<Number,Number> linkcnts;
	protected HashMap<Number,ArrayList<Instance>> linkedinstances;
	protected ArrayList<String> conflicts;
	protected boolean isConsistent;
	
	public Instance(Configuration co, clews.data.Class c)
	{
		config = co;
		name = c.getName();
		type = c;
		links = new HashMap<Number, ArrayList<Link>>();
		linkcnts = new HashMap<Number,Number>();		
		linkedinstances = new HashMap<Number, ArrayList<Instance>>();
		isConsistent = false;
		conflicts = new ArrayList<String>();		
	}
	
	public int getId()
	{
		return id;
	}
	 
	public void applyChanges()
	{
		boolean conflict=false;
		boolean isOr;
		int orCnt;
		String message;
		conflicts = new ArrayList<String>();
		
		orCnt = 0;
		isOr = (this.getType().getMinPartner() > 0 || this.getType().getMaxParnter() != Environment.CARDINALITY_STAR);
		if (isOr)
			isOr = true;
		for (AssociationEnd ae : type.getEnds())
		{
			message = "";
			conflict = false;
			if (ae.type == AssociationAttribute.UNIQUE)
			{
				if (isOr && ae.getOther().getMin() == ae.getOther().getMax() && 
						ae.getMin() != ae.getMax())
				{
					if (linkedinstances.containsKey(ae.assclass.getId()))
						orCnt++;
				}
				else
				{
					ArrayList<clews.data.Instance> tmp = new ArrayList<Instance>();
					ArrayList<Instance> insts=null;
					int lc = 0;
					if (linkedinstances.containsKey(ae.assclass.getId()))
					{
						insts = linkedinstances.get(ae.assclass.getId());
						lc = insts.size();
					}				
					if (lc < ae.getMin()){
							conflict = true;
							message = ""+lc+getCardString(lc," link");
					}
					if (lc > 0)
					{		 
						for (Instance i : insts){
								if (!tmp.contains(i))					
									tmp.add(i);
						}
						lc = tmp.size();
						if (lc < ae.getMin() || (ae.getMax() != Environment.CARDINALITY_STAR && lc > ae.getMax()))
						{
							conflict = true;
							message = ""+lc+getCardString(lc," unique object");
						}
					}
				}
			}
			else if (ae.type == AssociationAttribute.NON_UNIQUE)
			{
				int lc = 0;
				if (linkedinstances.containsKey(ae.assclass.getId()))
				{
					ArrayList<Instance> insts = linkedinstances.get(ae.assclass.getId());
					lc = insts.size();
				}
				conflict = (lc < ae.getMin() || (ae.getMax() != Environment.CARDINALITY_STAR && lc > ae.getMax()));
				if (conflict)
					message = ""+lc+getCardString(lc," link");
			} 
			else if (ae.type == AssociationAttribute.ALL_SAME)
			{
				ArrayList<Instance> insts=null;
				int lc = 0;
				if (linkedinstances.containsKey(ae.assclass.getId()))
				{
					insts = linkedinstances.get(ae.assclass.getId());
					lc = insts.size();
				}				
				if (lc < ae.getMin() || (ae.getMax() != Environment.CARDINALITY_STAR && lc > ae.getMax())) {
						conflict = true;
						message = ""+lc+getCardString(lc," link");
				}
				if (lc > 0)
				{
					Instance master = insts.get(0);
					for (Instance i : insts)
						if (i != master){
							conflict = true;
							message = "Not all the same";
							break;
						}
				}
			}			
			if (conflict)			
				conflicts.add(getConflicString(ae, message));
		}
		if (isOr)
			System.out.println(getName()+"  "+orCnt);
		if (isOr && (orCnt < getType().getMinPartner() || 
				(getType().getMaxParnter() != Environment.CARDINALITY_STAR && orCnt > getType().getMaxParnter())))
		{
			conflict = true;
			message = getType().getName()+" defines or["+getType().getMinPartner()+".."+Utils.toStarNum(getType().getMaxParnter())+"]";
			conflicts.add(message+"; Current: "+orCnt);
		}

		if (view != null) //TODO
			view.setPositionChanged();

//		System.out.println("Conflicts for "+ this.getName() +"> "+conflicts.size());
		isConsistent = (conflicts.size() == 0);
	}
	
	protected String getCardString(int cnt, String str)
	{
		return str+(cnt>1?"s":"");
	}
	
	public String getConflicString(AssociationEnd ae, String message)
	{
		String ret = ae.assoc.getName()+": ["+ae.getMin()+".."+ae.getMax()+"] " +ae.type.name()+ " to "+ae.assclass.getName();
		if (message != null && message.length() > 0)
			ret = ret +"; Current: "+message;
		return ret;		
	}
	
	public String getConflictString(String message)
	{
		return message;
	}
	
	public String reportConflicts()
	{
		String ret="";
		if (!isConsistent())
		{
			ret = "<html>";
			for (String c : conflicts)
			{
				ret = ret + c +"<br>";
			}
		}
		return ret.replace("-1", "*");
	}
	
	public boolean isConsistent()
	{		
		return isConsistent;
	}
	
	public void addLink(Link l, Instance i2)
	{
		if (!links.containsKey(i2.getId()))
			links.put(i2.getId(), new ArrayList<Link>());
		if (!linkcnts.containsKey(i2.getId()))
			linkcnts.put(i2.getId(), 0);
		if (!linkedinstances.containsKey(i2.getType().getId()))
			linkedinstances.put(i2.getType().getId(), new ArrayList<Instance>());
		links.get(i2.getId()).add(l);
		linkcnts.put(i2.getId(),linkcnts.get(i2.getId()).intValue()+1);
		linkedinstances.get(i2.getType().getId()).add(i2);
	}
	
	public void removeLink(Link li, Instance i2, int cnt)
	{
		if (links.containsKey(i2.getId()))
		{
			for (Link l : links.get(i2.getId()))
			{
				if (l.getView() != null)
				{
					if (l.left == i2 && l.getView().getLeftCnt() > cnt)
						l.getView().decreaseLeft();
					if (l.right == i2 && l.getView().getRightCnt() > cnt)
						l.getView().decreaseRight();
				}
			}
			links.get(i2.getId()).remove(li);
		}
		int ncnt = 0;
		if (linkcnts.containsKey(i2.getId()))
		{
			ncnt = linkcnts.get(i2.getId()).intValue()-1;
			linkcnts.put(i2.getId(),ncnt);
		}
		if (linkedinstances.containsKey(i2.getType().getId()))
			linkedinstances.get(i2.getType().getId()).remove(i2);
	}
	
	public int getNumberOfLinks(Instance i2)
	{
		if (!linkcnts.containsKey(i2.getId()))
			return 0;
		return linkcnts.get(i2.getId()).intValue();
	}
	
	
	public HashMap<Number,ArrayList<Link>> getLinks()
	{
		return links;
	}
	
	public Configuration getConfiguration()
	{
		return config;
	}
	
	public clews.data.Class getType()
	{
		return type;
	}
		
	public String getName()
	{
		return name;
	}
	
	public void setName(String n)
	{
		name = n;
		if (view != null)
			view.setSizeFromName();
	}
}
