package clews.data;

import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import net.sf.javailp.Constraint;
import net.sf.javailp.OptType;
import net.sf.javailp.Problem;
import net.sf.javailp.Result;
import clews.MainFrame;
import clews.env.Environment;
import clews.export.SpecificationILPExport;
import clews.gui.view.SpecificationView;

public class Specification 
extends DataObject<SpecificationView>
{
	protected ArrayList<Class> classes;
	protected ArrayList<Association> associations;
	protected ArrayList<clews.data.Constraint> constraints;
	protected boolean isConsistent;
	protected Result minimalModel;
	protected String path;

	
	protected HashMap<Class, Number> cmap = new HashMap<Class, Number>();
	protected HashMap<Number, Class> nmap = new HashMap<Number, Class>();
	protected HashMap<String, Class> smap = new HashMap<String, Class>();

	
	public Specification()
	{
		isConsistent = true;
		classes = new ArrayList<Class>();
		associations = new ArrayList<Association>();
		constraints = new ArrayList<clews.data.Constraint>();
	}
	
	public Result getMinimalModel()
	{
		return minimalModel;
	}
	
	public String getPath()
	{
		return path;
	}
	
	public void setPath(String p)
	{
		path = p;
	}
	
	public Class addNewClass()
	{
		return addNewClass(Environment.NO_CLASS_ID);
	}
	
	public boolean isConsistent()
	{
		return isConsistent;
	}
	
	public Class addNewClass(int id)
	{
		Class c = new Class(this,id);
		classes.add(c);
		return c;
	}
	
	public clews.data.Constraint addNewConstraint()
	{
		clews.data.Constraint c = new clews.data.Constraint(this);
		constraints.add(c);
		return c;
	}
	
	public void delete(clews.data.Constraint c)
	{
		c.removeFromAssociations();
		constraints.remove(c);
	}
	
	public ArrayList<clews.data.Constraint> getConstraints()
	{
		return constraints;
	}
	
	public ArrayList<Association> delete(Class c)
	{
		ArrayList<Association> todel = new ArrayList<Association>();
		for (Association a : associations)
			if (a.getLeftClass() == c || a.getRightClass() == c)
				todel.add(a);
		for (Association a : todel)
			delete(a);
		classes.remove(c);
		return todel;
	}
	
	public void delete(Association a)
	{
		associations.remove(a);
	}
	
	public Class getClassById(int id)
	{
		for (Class c : classes)
			if (c.id == id)
				return c;
		return null;
	}
	
	public Association getAssociationById(int id)
	{
		for (Association a : associations)
			if (a.id == id)
				return a;
		return null;
	}
	
	public Association getAssociationForName(String name)
	{
		for (Association a : associations)
			if (a.getName().equals(name))
				return a;
		return null;
	}

	public Association associate(Class c1, Class c2)
	{
  	        return associate(c1, c2, true);
	}

	public Association associate(Class c1, Class c2, boolean applyChanges)
	{
		for (Association a : associations) // TODO just binary
			if ((a.getLeft().assclass == c1 && a.getRight().assclass == c2) ||
				(a.getLeft().assclass == c2 && a.getRight().assclass == c1))
				return null;
		
		Association a = new Association(c1,c2);
		associations.add(a);
		c1.addAssociationEnd(a.getRight());
		c2.addAssociationEnd(a.getLeft());
		if (applyChanges)
 		        applyChanges();
		return a;
	}

	public Association associate(ArrayList<AssociationEnd> ends)
	{
	        return associate(ends,true);
	}

	public Association associate(ArrayList<AssociationEnd> ends, boolean applyChanges)
	{
		Association a= new Association(ends);
		if (a.isBinary())
		{
			associations.add(a);
			a.getLeft().assclass.addAssociationEnd(a.getRight());
			a.getRight().assclass.addAssociationEnd(a.getLeft());
			if (applyChanges)
		    	        applyChanges();
		}
		else
		{
			//TODO
		}
		return a;
	}
	
	public void applyChanges()
	{
		for (Class c : classes)
			c.setIsConsistent(true);
		for (Association a : associations)
			a.setIsConsistent(true);
		isConsistent = check();
		if (!isConsistent)
		{
			for (Association a : associations)
				if (!a.getLeft().assclass.isConsistent() && !a.getRight().assclass.isConsistent())
					a.setIsConsistent(false);
		}
		else
		{
			SpecificationILPExport ilp = new SpecificationILPExport(this);
			ilp.export();
			ilp.setObjective(OptType.MIN); 
			minimalModel = ilp.solve();
			if (minimalModel != null){
				for (String var : smap.keySet())
				{
					if (minimalModel.containsVar(var))
					{
						//System.out.println(var+"="+r.get(var).intValue());
						smap.get(var).setMinModelObj(minimalModel.get(var).intValue());
					}				
					
				}
			}
		}
		
		if (MainFrame.getInstance() != null && MainFrame.getInstance().getDomain() != null)
			for (Configuration c : MainFrame.getInstance().getDomain().getConfigurations())
				for (Instance i : c.getInstances())
					i.applyChanges();

	}
	
	public Configuration createMinimalConfiguration()
	{		
		Configuration config=null;
		applyChanges();
		if (isConsistent){
			config = MainFrame.getInstance().getDomain().addNewConfiguration(); // TODO
			if (minimalModel != null){
				int y = 100;
				for (String var : smap.keySet())
				{
					if (minimalModel.containsVar(var))
					{
						Class c = getClassForName(var.replace("|", ""));
						int cnt = minimalModel.get(var).intValue();
						for (int i=0;i<cnt;i++) 
							config.getView().addDropInstance(c,new Point((500-200*cnt/2)+i*200,y));							
					}				
					y += 150;
				}
				
				for (Association a: associations)
					config.addMinLinks(a.left,a.right);
			}
		}
		return config;
	}
	
	public clews.data.Class getClassForName(String name)
	{
		for (clews.data.Class c : classes)
			if (c.getName().equals(name))
				return c;
		return null;
	}
	
	protected void markPath(int i, int[][] pred)
	{
		int a,b,cur;
		cur = a = b = i;
		do
		{
			nmap.get(cur).setIsConsistent(false);
			System.out.println("-> "+cur);
			if (pred[a][b] == cur)
				break;
			cur = pred[a][b];
			b = cur;					
		}while (cur != i && cur != -1);
		
		cur = a = b = i;
		do
		{
			nmap.get(cur).setIsConsistent(false);
			System.out.println("-> "+cur);
			if (pred[a][b] == cur)
				break;
			cur = pred[a][b];
			a = cur;					
		}while (cur != i && cur != -1);
		
		/*
		int a,b,cur;
		a = b = i;
		do
		{
			cur = pred[a][b];
			System.out.println("-> "+cur);
			b = cur;					
		}while (cur != i);
		*/
	}
	
	protected boolean check()
	{
		int nodes = classes.size();
		double[][] adjm = new double[nodes][nodes];
		int[][] pred = new int[nodes][nodes];
		boolean[][] orig = new boolean[nodes][nodes];
		cmap = new HashMap<Class, Number>();
		nmap = new HashMap<Number, Class>();
		smap = new HashMap<String, Class>();
		
		for (int i=0;i<nodes;i++)
		{
			smap.put(SpecificationILPExport.makeVar(classes.get(i).getName()), classes.get(i));
			cmap.put(classes.get(i), i);
			nmap.put(i, classes.get(i));
//			System.out.println(i+": "+classes.get(i));
			for (int j=0;j<nodes;j++)
			{
				adjm[i][j] = Integer.MAX_VALUE;
				pred[i][j] = -1;
				orig[i][j] = false;
			}
		}
		
		SpecificationILPExport ilp = new SpecificationILPExport(this);
		Problem p = ilp.export();
		for (Constraint c : p.getConstraints())
			if (c.getRhs().intValue() == 0 && c.getLhs().size() == 2)
			{
				int a=0,b=1;
				if (c.getLhs().get(0).getCoefficient().intValue() < 0)
				{
					a = 1;
					b = 0;
				}
				int x = cmap.get(smap.get(c.getLhs().get(a).getVariable())).intValue();
				int y = cmap.get(smap.get(c.getLhs().get(b).getVariable())).intValue();
				//System.out.println(c.getLhs().get(a).getCoefficient() + "  --  "+c.getLhs().get(b).getCoefficient());
				orig[x][y] = true;
				if (c.getLhs().get(a).getCoefficient().doubleValue() == 0)
					adjm[x][y] = Integer.MAX_VALUE;
				else
					adjm[x][y] = -c.getLhs().get(b).getCoefficient().doubleValue() /
				c.getLhs().get(a).getCoefficient().doubleValue();
				//System.out.println("++ "+adjm[x][y]);
			}		
/*
    	for (int ii = 0; ii < nodes; ii++)
		{
			for (int jj = 0; jj < nodes; jj++)
				System.out.print(adjm[ii][jj]+"\t");
			System.out.println();
		}
*/		
		
		for (int k = 0; k < nodes; k++)
            for (int i = 0; i < nodes; i++)
                for (int j = 0; j < nodes; j++)
                    if (adjm[i][k] != Integer.MAX_VALUE && adjm[k][j] != Integer.MAX_VALUE &&
                    	!orig[i][j] && adjm[i][k] * adjm[k][j] < adjm[i][j])
                    {
                        adjm[i][j] = adjm[i][k] * adjm[k][j];
                        pred[i][j] = k;
                      // System.out.println(i+"->"+j+" via "+k+" mit "+adjm[i][j]);
                        if (i == j && adjm[i][j] < 1)
                        {
                        	System.out.println();
                        	for (int ii = 0; ii < nodes; ii++)
                    		{
                    			for (int jj = 0; jj < nodes; jj++)
                    				System.out.print(adjm[ii][jj]+"\t");
                    			System.out.println();
                    		}
                        
                        	markPath(i,pred);
            				return false;
                        }                    
                    }
		
		
		/*
		 * Check if cycle in complete graph
		 */
		for (int k = 0; k < nodes; k++)
            for (int i = 0; i < nodes; i++)
                for (int j = 0; j < nodes; j++)
                    if (adjm[i][k] != Integer.MAX_VALUE && adjm[k][j] != Integer.MAX_VALUE &&
                    	adjm[i][k] * adjm[k][j] < adjm[i][j])
                    {
                        adjm[i][j] = adjm[i][k] * adjm[k][j];
                        pred[i][j] = k;
                      // System.out.println(i+"->"+j+" via "+k+" mit "+adjm[i][j]);
                        if (i == j && adjm[i][j] < 1)
                        {
                        	System.out.println();
                        	for (int ii = 0; ii < nodes; ii++)
                    		{
                    			for (int jj = 0; jj < nodes; jj++)
                    				System.out.print(adjm[ii][jj]+"\t");
                    			System.out.println();
                    		}
                        
                        	markPath(i,pred);
            				return false;
                        }                    
                    }
		
		
		for (int i = 0; i < nodes; i++)
		{
		//	System.out.print(i+"="+adjm[i][i]+", ");
			if (adjm[i][i] > 0 && adjm[i][i] < 1)
			{
				markPath(i,pred);
				return false;
			}
			
		}
		return true;

	}
		
	public ArrayList<Class> getClasses()
	{
		return classes;
	}
	
	public ArrayList<Association> getAssociations()
	{
		return associations;
	}
}
