package clews.env;

import java.util.ArrayList;

import clews.data.Instance;
import clews.data.Link;

public class Graph {

	protected ArrayList<Node> nodes;
	protected ArrayList<Edge> edges;
	protected Node source, sink;
	
	
	public void init(int l, ArrayList<Instance> left, ArrayList<clews.data.Instance> right, ArrayList<Link> links){
		nodes = new ArrayList<Node>();
		edges = new ArrayList<Edge>();
		
		l = l - links.size();
		
		source = new Node(null,l);
		sink = new Node(null,l);
		nodes.add(source);
		nodes.add(sink);	
		
		
		Node n; Edge e;
		for (Instance i : left){
			nodes.add((n = new Node(i,0)));
			edges.add((e=new Edge(source,n,1,l/left.size(),(l+left.size()-1)/left.size())));
			source.out.add(e);
			e.back = new Edge(n,source,0,0,0);
			e.back.back = e;
			n.out.add(e.back);
		}
		
		for (Instance i : right){
			nodes.add((n = new Node(i,0)));
			edges.add((e=new Edge(n,sink,1,l/right.size(),(l+right.size()-1)/right.size())));
			n.out.add(e);
			e.back = new Edge(sink,n,0,0,0);
			e.back.back = e;
			sink.out.add(e.back);
		}
		
		for (Instance i : left){
			for (Instance i2 : right){
				if (!exisiting(links,i,i2)){
					edges.add(e=new Edge(getNode(i),getNode(i2),1,0,1));
					getNode(i).out.add(e);
					e.back = new Edge(getNode(i2),getNode(i),0,0,0);
					e.back.back = e;
					getNode(i2).out.add(e.back);
				}
			}			
		}
		for (Edge ee : edges)
			System.out.println(ee.mincap + " " +ee.maxcap);
	}
	
	boolean exisiting(ArrayList<Link> ls, Instance i1, Instance i2){
		for (Link l : ls){
			if ((l.getLeft() == i1 && l.getRight() == i2) ||
				(l.getLeft() == i2 && l.getRight() == i1))
				return true;			
		}
		return false;
	}
	
	Node getNode(Instance i){
		for (Node n : nodes){
			if (n.rep == i)
				return n;
		}
		return null;
	}
	
	Edge getEdgeForInstances(Instance from, Instance to){
		for (Edge e  : edges)
			if (e.from.rep == from && e.to.rep == to)
				return e;		
		return null;
	}
	
	public boolean isFlow(Instance from, Instance to){
		Edge e = getEdgeForInstances(from, to);
		if (e != null)
			return e.flow != 0;
		return false;
	}
	
	public boolean findFeasible(){
		while (source.req > 0){
			ArrayList<Edge> p = findPath(source,sink); 
			if (p == null)
				return false;
			for (Edge e : p){
				e.flow++;
				e.from.req--;
				e.to.req++;
				e.back.flow--;
			}
		}
		return true;
	}
	
	ArrayList<Edge> findPath(Node from, Node to){
		return findPathRecursive(from,to,new ArrayList<Node>(),new ArrayList<Edge>());
	}
	
	ArrayList<Edge> findPathRecursive(Node cur, Node dest, ArrayList<Node> visited, ArrayList<Edge> path){
		ArrayList<Edge> ret;
		visited.add(cur);
		if (cur == dest)
			return path;
		for (Edge e : cur.out)
			if (!visited.contains(e.to) && e.flow+1 <= e.maxcap){
				path.add(e);
				if ((ret=findPathRecursive(e.to, dest, visited, path)) != null)
					return ret;
				path.remove(e);
			}
		return null;
	}
		
	class Node
	{
		int req;
		Instance rep;
		ArrayList<Edge> out;
		
		public Node(Instance i, int r){
			rep = i;
			req = r;
			out = new ArrayList<Edge>();
		}
	}
	
	class Edge
	{
		int weight, flow, mincap, maxcap;		
		
		Node from;
		Node to;
		Edge back;
		
		public Edge(Node f, Node t, int w,int min, int max){
			from = f;			
			to = t;
			weight = w;
			flow = 0;
			mincap = min;
			maxcap = max;
//			back = 
		}		
	}
}
