package clews.export;

import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Node;
import clews.data.Configuration;
import clews.data.Instance;
import clews.data.Link;
import clews.env.Environment;
import clews.gui.view.InstanceView;

public class ConfigurationXMLExport 
extends XMLExport
{
	protected Configuration configuration;
	
	public ConfigurationXMLExport(Configuration c)	{
		configuration = c;
	}
	
	public boolean saveToDisk(String path)
	{
		return save(path,false);
	}
	
	public boolean save(String path, boolean isTmp)
	{
		try
		{
	        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();       
	        doc = builder.newDocument();
	        doc.setXmlStandalone(true);			

			Node main = doc.createElement("configuration");

			if (!isTmp && configuration.getPath() != null)
				path = configuration.getPath();
			else
			{
				path = path.replace("."+Environment.DOMAIN_ENDING, "."+configuration.getName()+"."+Environment.CONFIGURATION_ENDING);
				configuration.setPath(path);
			}

			if (configuration.getView() != null)
			{
				AffineTransform trafo = configuration.getView().getDiagram().getTransform();
				addAttribute(main, "scale", ""+trafo.getScaleX());
				addAttribute(main, "transx", ""+trafo.getTranslateX());
				addAttribute(main, "transy", ""+trafo.getTranslateY());
			}
			doc.appendChild(main);
			
			ArrayList<Instance> instances = configuration.getInstances();
			for (Instance i : instances)
			{
				Node cur = doc.createElement("instance");
				main.appendChild(cur);
				addAttribute(cur, "id", ""+i.getId());
				addAttribute(cur, "name", i.getName());
				addAttribute(cur, "instanceOf", i.getType().getName());
				InstanceView view = i.getView();
				if (view != null)
				{
					addAttribute(cur, "x", ""+(int)view.getCenter().getX());
					addAttribute(cur, "y", ""+(int)view.getCenter().getY());
					addAttribute(cur, "radius", ""+view.getRadius());
				}
			}

			ArrayList<Link> links = configuration.getLinks();			
			for (Link l : links)
			{
				Node cur = doc.createElement("link");
				if (l.getDescription() != null)
					addAttribute(cur, "description", ""+l.getDescription());
				main.appendChild(cur);				
				for (Instance i : l.getInstances())
				{
					Node end = doc.createElement("linkend");
					addAttribute(end, "instance", ""+i.getId());
					cur.appendChild(end);
				}
			}
						
			toDisk(path);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return false;
		}
		return true;
	}	
}
