package clews.export;

import java.awt.Point;
import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.XMLReaderFactory;

import clews.data.Association;
import clews.data.Configuration;
import clews.data.DataObject;
import clews.data.Instance;
import clews.data.Link;
import clews.data.Specification;
import clews.gui.view.ConfigurationView;

public class ConfigurationXMLImport {
	
	protected Configuration configuration;
	protected ConfigurationView configurationView;
	protected Specification specification;
	protected ArrayList<Instance> ends;
	protected String description;
	
	public ConfigurationXMLImport(Specification s)
	{
		specification = s;
	}	
	
	public Configuration loadFromDisk(String path)
	{
		configuration = null;
		try
		{
			XMLReader reader = XMLReaderFactory.createXMLReader();
			reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
			
			DiagramSaxParser sax = new DiagramSaxParser();			
			sax.setParent(reader);
			sax.parse(path);
			setIds();
			configuration.applyChanges();
		} 
		catch (Exception ex)
		{
			ex.printStackTrace();
			return null;
		}
		return configuration;
	}
	
	public Configuration getConfiguration()
	{
		return configuration;
	}
	
	
	class DiagramSaxParser extends XMLFilterImpl
	{		
		public void startDocument() throws SAXException {
			super.startDocument();			
		}
		
		public void endDocument() throws SAXException {
			super.endDocument();
		}
						
		public void startElement(String namespaceURI, String localName, String qualifiedName, Attributes atts) throws SAXException {
			if (localName.equals("configuration"))
			{
				configuration = new Configuration(specification);
				configurationView = new ConfigurationView(configuration);
				configuration.setView(configurationView);			
				if (atts.getIndex("scale") != -1){
					configurationView.getDiagram().setTransformParameter(
							Double.parseDouble(atts.getValue("scale")),
							Double.parseDouble(atts.getValue("transx")),
							Double.parseDouble(atts.getValue("transy"))
							);
				}
			}
			else if (localName.equals("instance"))
			{
				clews.data.Class c = specification.getClassForName(atts.getValue("instanceOf"));
				if (c != null)
				{
					Point p = new Point();
					if (atts.getIndex("x") != -1)
						p = new Point(Integer.parseInt(atts.getValue("x")),Integer.parseInt(atts.getValue("y")));
					Instance i = configurationView.addDropInstance(c,p);
					i.setName(atts.getValue("name"));
					i.setId(Integer.parseInt(atts.getValue("id")));
				}
				else
				{
					throw new SAXException("invalid configuration for specification");
				}
			}
			else if (localName.equals("link"))
			{
				description = null;
				if (atts.getIndex("description") != -1)
					description = atts.getValue("description");
				ends = new ArrayList<Instance>();				
			}
			else if (localName.equals("linkend"))
			{
				ends.add(configuration.getInstanceById(Integer.parseInt(atts.getValue("instance"))));
			}

			super.startElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase(), atts);
		}

		public void endElement(String namespaceURI, String localName, String qualifiedName) throws SAXException {
			
			if (localName.equals("link"))
			{
				Link l = configuration.link(ends);
				if (description != null)
					l.setDescription(description);
				configurationView.addNewLinkView(l);
			}
			super.endElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase());
		}
		
		public void characters(char[] ch, int start, int length) throws SAXException
		{
			super.characters(ch, start, length);
		}		
	}
	
	protected void setIds()
	{
		int max;
		max = Integer.MIN_VALUE;
		for (Instance i : configuration.getInstances())
			max = Math.max(max, i.getId());
		DataObject.setId(Instance.class, max);
		
		max = Integer.MIN_VALUE;
		for (Link l : configuration.getLinks())
			max = Math.max(max, l.getId());
		DataObject.setId(Link.class, max);	

	}
}
