package clews.export;

import java.io.File;
import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.XMLReaderFactory;

import clews.Domain;
import clews.MainFrame;
import clews.data.Association;
import clews.data.AssociationEnd;
import clews.data.AssociationAttribute;
import clews.data.Configuration;
import clews.data.Constraint;
import clews.data.DataObject;
import clews.data.Specification;
import clews.export.SpecificationXMLImport.DiagramSaxParser;
import clews.gui.view.ClassView;
import clews.gui.view.ConstraintView;
import clews.gui.view.SpecificationView;

public class DomainXMLImport {

	protected ArrayList<Configuration> configurations;
	protected Specification specification;
	protected String dir;
	protected String active;
	protected Domain domain;

	public ArrayList<Configuration> getConfigurations()
	{
		return configurations;
	}
	
	public Specification getSpecification()
	{
		return specification;
	}
	
	public String getActive()
	{
		return active;
	}
	
	public Domain getDomain()
	{
		return domain;
	}
	
	public boolean loadFromDisk(String path)
	{
		try
		{
			XMLReader reader = XMLReaderFactory.createXMLReader();
			reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
			
			dir = (new File(path)).getParent();
			if (dir == null)
				dir = "";
			active = null;
			
			DiagramSaxParser sax = new DiagramSaxParser();			
			sax.setParent(reader);
			sax.parse(path);
			setIds();
			
			 domain = new Domain();
			 domain.newDomain(false);
			 domain.setPath(path);
			 domain.setConfigurations(getConfigurations());
			 domain.setSpecification(getSpecification());
		} 
		catch (Exception ex)
		{
			ex.printStackTrace();
			return false;
		}
		return true;
	}	

	class DiagramSaxParser extends XMLFilterImpl
	{		
		public void startDocument() throws SAXException {
			super.startDocument();			
		}
		
		public void endDocument() throws SAXException {
			super.endDocument();
		}
						
		public void startElement(String namespaceURI, String localName, String qualifiedName, Attributes atts) throws SAXException {
			if (localName.equals("domain"))
			{
				if (atts.getIndex("active") != -1)
					active = atts.getValue("active");
			}
			if (localName.equals("specification"))
			{
				SpecificationXMLImport sxi = new SpecificationXMLImport();
				specification = sxi.loadFromDisk(dir+File.separator+atts.getValue("path"));
			}
			else if (localName.equals("configurations"))
			{
				configurations = new ArrayList<Configuration>();
			}
			else if (localName.equals("configuration"))
			{
				ConfigurationXMLImport cxi = new ConfigurationXMLImport(specification);
				Configuration c = cxi.loadFromDisk(dir+File.separator+atts.getValue("path"));
				if (c != null)
				{
					c.setName(atts.getValue("name"));
					configurations.add(c);
				}
			}
			super.startElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase(), atts);
		}

		public void endElement(String namespaceURI, String localName, String qualifiedName) throws SAXException {
			
			super.endElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase());
		}
		
		public void characters(char[] ch, int start, int length) throws SAXException
		{
			super.characters(ch, start, length);
		}		
	}
	
	protected void setIds()
	{
		int max;		
		max = Integer.MIN_VALUE;
		for (Configuration c : configurations)
			max = Math.max(max, c.getId());
		DataObject.setId(Configuration.class, max);

	}
}
