package clews.export;

import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Node;

import clews.data.Association;
import clews.data.AssociationEnd;
import clews.data.Configuration;
import clews.data.Constraint;
import clews.data.Specification;
import clews.data.Constraint.ConstraintPart;
import clews.gui.view.ClassView;

public class SpecificationXMLExport 
extends XMLExport 
{
	protected Specification specification;
	
	public SpecificationXMLExport(Specification s)	{
		specification = s;
	}
	
	public boolean saveToDisk(String path)
	{
		return save(path,false);
	}
	
	public boolean save(String path, boolean isTmp)
	{
		try
		{
	        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();       
	        doc = builder.newDocument();
	        doc.setXmlStandalone(true);			

			Node main = doc.createElement("specification");			
	        
			specification.setPath(path);
			if (specification.getView() != null)
			{
				AffineTransform trafo = specification.getView().getDiagram().getTransform();
				addAttribute(main, "scale", ""+trafo.getScaleX());
				addAttribute(main, "transx", ""+trafo.getTranslateX());
				addAttribute(main, "transy", ""+trafo.getTranslateY());
			}
			doc.appendChild(main);
			
			ArrayList<clews.data.Class> classes = specification.getClasses();
			for (clews.data.Class c : classes)
			{
				Node cur = doc.createElement("class");
				ClassView view = c.getView();
				main.appendChild(cur);
				addAttribute(cur, "id", ""+c.getId());
				addAttribute(cur, "name", c.getName());
				addAttribute(cur, "minobj", ""+c.getMinObj());
				addAttribute(cur, "weight", ""+c.getWeight());
				addAttribute(cur, "maxobj", ""+c.getMaxObj());
				addAttribute(cur, "minpartner", ""+c.getMinPartner());
				addAttribute(cur, "maxpartner", ""+c.getMaxParnter());
				Rectangle r;
				if (view != null)
					r = view.getBounds();
				else
					r = new Rectangle();					
				addAttribute(cur, "x", ""+r.x);
				addAttribute(cur, "y", ""+r.y);
				addAttribute(cur, "width", ""+r.width);
				addAttribute(cur, "height", ""+r.height);
			}

			ArrayList<Association> assocs = specification.getAssociations();			
			for (Association a : assocs)
			{
				Node cur = doc.createElement("association");
				addAttribute(cur, "name", ""+a.getName());
				addAttribute(cur, "id", ""+a.getId());
				main.appendChild(cur);				
				for (AssociationEnd e : a.getEnds())
				{
					Node end = doc.createElement("associationend");
					addAttribute(end, "assclass", ""+e.assclass.getId());
					addAttribute(end, "type", ""+e.type);
					addAttribute(end, "min", ""+e.getMin());
					addAttribute(end, "max", ""+e.getMax());
					cur.appendChild(end);
				}
			}
			
			for (Constraint c : specification.getConstraints())
			{
				Node cur = doc.createElement("constraint");
				addAttribute(cur, "id", ""+c.getId());
				addAttribute(cur, "name", c.getName());
				addAttribute(cur, "isactive", ""+c.isActive());
				main.appendChild(cur);
				for (Constraint.ConstraintPart cp : c.getLhs())
				{
					Node part = doc.createElement("constraintpartlhs");
					addAttribute(part, "association", ""+cp.association.getId());
					addAttribute(part, "inv", ""+cp.inv);
					cur.appendChild(part);
				}
				for (Constraint.ConstraintPart cp : c.getRhs())
				{
					Node part = doc.createElement("constraintpartrhs");
					addAttribute(part, "association", ""+cp.association.getId());
					addAttribute(part, "inv", ""+cp.inv);
					cur.appendChild(part);
				}
			}		
			
			
			toDisk(path);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return false;
		}
		return true;
	}	
	
}
