package clews.export;

import java.util.ArrayList;



import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.XMLReaderFactory;
import clews.data.Association;
import clews.data.AssociationEnd;
import clews.data.AssociationAttribute;
import clews.data.Configuration;
import clews.data.Constraint;
import clews.data.DataObject;
import clews.data.Specification;
import clews.gui.view.ClassView;
import clews.gui.view.ConstraintView;
import clews.gui.view.SpecificationView;

public class SpecificationXMLImport {
	
	protected Specification specification;
	protected SpecificationView specificationView;
	protected ArrayList<AssociationEnd> ends;
	protected String associationName;
	protected int associationId;
	protected Constraint constraint;

	public Specification loadFromDisk(String path)
	{
		specification = null;
		try
		{
			XMLReader reader = XMLReaderFactory.createXMLReader();
			reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
			
			DiagramSaxParser sax = new DiagramSaxParser();			
			sax.setParent(reader);
			sax.parse(path);
			setIds();
			specification.applyChanges();
		} 
		catch (Exception ex)
		{
			ex.printStackTrace();
			return null;
		}
		return specification;
	}
	
	public Specification getSpecification()
	{
		return specification;
	}

	
	class DiagramSaxParser extends XMLFilterImpl
	{		
		public void startDocument() throws SAXException {
			super.startDocument();			
		}
		
		public void endDocument() throws SAXException {
			super.endDocument();
		}
						
		public void startElement(String namespaceURI, String localName, String qualifiedName, Attributes atts) throws SAXException {
			if (localName.equals("specification"))
			{
				specification = new Specification();
				specificationView = new SpecificationView(specification);
				specification.setView(specificationView);					
				if (atts.getIndex("scale") != -1){
				specificationView.getDiagram().setTransformParameter(
						Double.parseDouble(atts.getValue("scale")),
						Double.parseDouble(atts.getValue("transx")),
						Double.parseDouble(atts.getValue("transy"))
						);
				}
			}
			else if (localName.equals("class"))
			{
				clews.data.Class c = specification.addNewClass(Integer.parseInt(atts.getValue("id")));
				c.setName(atts.getValue("name"));
				c.setMinObj(Integer.parseInt(atts.getValue("minobj")));				
				ClassView view = specificationView.addNewClassView(c);
				view.setLocation(
						Integer.parseInt(atts.getValue("x")),
						Integer.parseInt(atts.getValue("y"))
						);
				view.getBounds().setSize(
						Integer.parseInt(atts.getValue("width")),
						Integer.parseInt(atts.getValue("height"))
						);
				try{
					c.setWeight(Integer.parseInt(atts.getValue("weight")));
					c.setMaxObj(Integer.parseInt(atts.getValue("maxobj")));
					c.setMinPartner(Integer.parseInt(atts.getValue("minpartner")));
					c.setMaxPartner(Integer.parseInt(atts.getValue("maxpartner")));
				}catch(Exception e){}
			}
			else if (localName.equals("association"))
			{
				ends = new ArrayList<AssociationEnd>();
				associationName = atts.getValue("name");
				associationId = Integer.parseInt(atts.getValue("id"));
			}
			else if (localName.equals("associationend"))
			{
				ends.add(new AssociationEnd(
						null, 
						AssociationAttribute.valueOf(atts.getValue("type")), 
						Integer.parseInt(atts.getValue("min")), 
						Integer.parseInt(atts.getValue("max")), 
						specification.getClassById(Integer.parseInt(atts.getValue("assclass")))
						));
			}
			else if(localName.equals("constraint"))
			{
				constraint = specification.addNewConstraint();
				constraint.setView(new ConstraintView(constraint));
				constraint.setId(Integer.parseInt(atts.getValue("id")));
				constraint.setName(atts.getValue("name"));
				constraint.setActive(Boolean.parseBoolean(atts.getValue("isactive")));				
			}else if (localName.equals("constraintpartlhs"))
			{
				constraint.addLhsPart(
					specification.getAssociationById(Integer.parseInt(atts.getValue("association"))), 
					Boolean.parseBoolean(atts.getValue("inv")));
			}else if (localName.equals("constraintpartrhs"))
			{
				constraint.addRhsPart(
						specification.getAssociationById(Integer.parseInt(atts.getValue("association"))), 
						Boolean.parseBoolean(atts.getValue("inv")));				
			}
			super.startElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase(), atts);
		}

		public void endElement(String namespaceURI, String localName, String qualifiedName) throws SAXException {
			
			if (localName.equals("association"))
			{
			        Association a = specification.associate(ends,false);
				a.setName(associationName);
				a.setId(associationId);
				specificationView.addNewAssociationView(a);
			}
			super.endElement(namespaceURI, localName.toLowerCase(),qualifiedName.toLowerCase());
		}
		
		public void characters(char[] ch, int start, int length) throws SAXException
		{
			super.characters(ch, start, length);
		}		
	}
	
	protected void setIds()
	{
		int max;
		max = Integer.MIN_VALUE;
		for (clews.data.Class c : specification.getClasses())
			max = Math.max(max, c.getId());
		DataObject.setId(clews.data.Class.class, max);
		
		max = Integer.MIN_VALUE;
		for (Association a : specification.getAssociations())
			max = Math.max(max, a.getId());
		DataObject.setId(Association.class, max);
				
		max = Integer.MIN_VALUE;
		for (Constraint c : specification.getConstraints())
			max = Math.max(max, c.getId());
		DataObject.setId(Constraint.class, max);
	}
}
