package clews.gui;

import java.util.ArrayList;


import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import clews.Domain;
import clews.data.Configuration;
import clews.data.Specification;

public class DomainTreeModel
implements TreeModel
{
	protected Domain domain;
	protected String root = "Domain";
	protected ArrayList<String> groups;
	
	public DomainTreeModel(Domain d)
	{
		domain = d;
		groups = new ArrayList<String>();
		groups.add("Specification");
		groups.add(getClassGroup());
		groups.add("Associations");
		groups.add("Constraints");
		groups.add("Configurations");
	}	

	public Specification getSpecification()
	{
		return domain.getSpecification();
	}
	
	public ArrayList<Configuration> getConfigurations()
	{
		return domain.getConfigurations();
	}
	
	public String getClassGroup()
	{
		return "Classes";
	}
	
	@Override
	public void addTreeModelListener(TreeModelListener l) {
	}

	@Override
	public Object getChild(Object parent, int index) {
		if (parent.toString().equals(root))
		{
			return groups.get(index);
		}
		else
		{
			int idx = groups.indexOf(parent);
			switch (idx)
			{
			case 1:
				return domain.getSpecification().getClasses().get(index);
			case 2:
				return domain.getSpecification().getAssociations().get(index);
			case 3:
				return domain.getSpecification().getConstraints().get(index);
			case 4:
				return domain.getConfigurations().get(index);
			}
		}
		return null;
	}

	@Override
	public int getChildCount(Object parent) {
		if (parent.toString().equals(root))
			return groups.size();
		else
		{
			int idx = groups.indexOf(parent);
			switch (idx)
			{
			case 1:
				return domain.getSpecification().getClasses().size();
			case 2:
				return domain.getSpecification().getAssociations().size();
			case 3:
				return domain.getSpecification().getConstraints().size();
			case 4:
				return domain.getConfigurations().size();
			}
		}
		return 0;
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) {
		if (parent.toString().equals(root))
			return groups.indexOf(child);
		else
		{
			int idx = groups.indexOf(parent);
			switch (idx)
			{
			case 1:
				return domain.getSpecification().getClasses().indexOf(child);
			case 2:
				return domain.getSpecification().getAssociations().indexOf(child);
			case 3:
				return domain.getSpecification().getConstraints().indexOf(child);				
			case 4:
				return domain.getConfigurations().indexOf(child);
			}
		}
		return -1;
	}

	@Override
	public Object getRoot() {
		return root;
	}

	@Override
	public boolean isLeaf(Object node) {
		if (node.toString().equals(root))
			return false;
		else
		{
			if (groups.indexOf(node) == 0)
				return true;
			return (groups.indexOf(node) == -1);// || groups.indexOf(node) >= 2);
		}
	}

	@Override
	public void removeTreeModelListener(TreeModelListener l) {
	}

	@Override
	public void valueForPathChanged(TreePath path, Object newValue) {
	}
}
