package clews.gui;

import java.awt.event.ActionEvent;

import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.*;

import net.sf.javailp.OptType;
import net.sf.javailp.Result;

import clews.Domain;
import clews.History;
import clews.MainFrame;
import clews.env.Environment;
import clews.env.Environment.ViewDetails;
import clews.export.DomainXMLExport;
import clews.export.DomainXMLImport;
import clews.export.SpecificationILPExport;
import clews.export.SpecificationXMLExport;
import clews.export.SpecificationXMLImport;


public class MainMenu 
extends JMenuBar
implements ActionListener
{
	private static final long serialVersionUID = -2579293378832746912L;
	protected MainFrame mainFrame;
	protected JMenuItem undoItem, redoItem;
		
	public MainMenu(MainFrame mf)
	{
		mainFrame = mf;
		Entry[] entries = new Entry[]{
				new Entry(new Entry[]{
						new Entry("File"),
						new Entry("New", new FileNewListener()),
						new Entry("Open", new FileOpenListener()),
						new Entry(""),
						new Entry("Save", new FileSaveListener(false)),
						new Entry("Save as", new FileSaveListener(true)),
						new Entry(""),
						new Entry(new Entry[]{
							new Entry("Export"),
							new Entry("as PNG", new FileExportImageListener("png",false)),
							new Entry("as GIF", new FileExportImageListener("gif",false)),
							new Entry("as EPS", new FileExportImageListener("eps",false))
						}),
						new Entry(new Entry[]{
								new Entry("Export Screen"),
								new Entry("as PNG", new FileExportImageListener("png",true)),
								new Entry("as GIF", new FileExportImageListener("gif",true)),
								new Entry("as EPS", new FileExportImageListener("eps",true))
							}),
						new Entry(new Entry[]{
								new Entry("Import"),
								new Entry("Existing Specification", new FileImportSpecifciation()),
								new Entry("Existing Configuration", new FileImportConfiguration())
						}),
						new Entry(""),
						new Entry("Exit", new FileExitListener())
						}),
				new Entry(new Entry[]{
						new Entry("Edit"),
						new Entry("Undo", new EditUndoListener()),
						new Entry("Redo", new EditRedoListener())
						}),
				new Entry(new Entry[]{
						new Entry("Domain"),
						new Entry("New Class",new DomainNewClassListener()),
						new Entry("New Constraint", new DomainNewConstraintListener()),
						new Entry(""),
						new Entry("New Configuration", new DomainNewConfigurationListener()),						
						new Entry("New Configuration from minimal model", new DomainNewConfigurationMinimalListener()),
						new Entry("Extend configuration to instance",new DomainExtendListener()),
						new Entry(""),
						new Entry("Check manually", new DomainCheckListener())
						}),
				new Entry(new Entry[]{
						new Entry("View"),
						new Entry("All Details",new ViewDetailsListener(ViewDetails.ALL)),
						new Entry("Some Details",new ViewDetailsListener(ViewDetails.SOME)),
						new Entry("No Details",new ViewDetailsListener(ViewDetails.NO)),
						new Entry("Custom Details",new ViewDetailsListener(ViewDetails.NO)),
						new Entry(""),
						new Entry("Reset",new ViewResetListener()),
						new Entry(""),
						new Entry("Zoom In",new ViewZoomInListener()),
						new Entry("Zoom Out",new ViewZoomOutListener()),
						new Entry(""),
						new Entry("Move Left",new ViewMoveLeftListener()),
						new Entry("Move Right",new ViewMoveRightListener()),
						new Entry("Move Up",new ViewMoveUpListener()),
						new Entry("Move Down",new ViewMoveDownListener())
						}),						
		};
		createMenu(entries,null);
		
		//ATTENTION
		undoItem = this.getMenu(1).getItem(0);
		redoItem = this.getMenu(1).getItem(1);
		refreshUnRedo();
	}
	
	public void createMenu(Entry[] entries, JMenu parent)
	{		
		for (int i=0;i<entries.length;i++)
		{
			if (entries[i].children == null)
			{
				if (i > 0){
					if (entries[i].label.equals(""))
					{
						parent.addSeparator();
					}
					else
					{
						JMenuItem item = new JMenuItem(entries[i].label);
						item.setIcon(entries[i].icon);
						item.addActionListener(entries[i].listener);
						item.addActionListener(this);
						parent.add(item);
					}
				}
			}
			else
			{
				JMenu curMenu = new JMenu(entries[i].children[0].label);
				createMenu(entries[i].children,curMenu);
				if (parent == null)
					add(curMenu);
				else
					parent.add(curMenu);
			}
		}	
	}
	
	public void refreshUnRedo()
	{
		undoItem.setText("Undo");
		redoItem.setText("Redo");
		if (mainFrame.getDomain() != null)
		{
			History h = mainFrame.getDomain().getHistory();
			undoItem.setEnabled(h.getUndoStack().size()>0);
			if (undoItem.isEnabled())
				undoItem.setText("Undo "+h.getUndoStack().peek());
			redoItem.setEnabled(h.getRedoStack().size()>0);
			if (redoItem.isEnabled())
				redoItem.setText("Redo "+h.getRedoStack().peek());				
		}
		else
		{
			undoItem.setEnabled(false);
			redoItem.setEnabled(false);			
		}		
	}
	
	class Entry
	{
		public String label;
		public Icon icon;
		public ActionListener listener;
		public Entry[] children;
		
		public Entry(String l)
		{
			this(l,null,null);
		}
		
		public Entry(Entry[] entries)
		{
			this(null,null,null);
			children = entries;
		}
		
		public Entry(String l, ActionListener al)
		{
			this(l,al,null);
		}
		
		public Entry(String l, ActionListener al, Icon i)
		{
			label = l;
			listener = al;
			icon = i;
			children = null;
		}
	}
	
	class EditUndoListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().getHistory().doUndo();
		}	
	}
	
	class EditRedoListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().getHistory().doRedo();
		}	
	}
	
	class DomainNewClassListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().addNewClass();
		}	
	}
	
	class DomainNewConfigurationListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().addNewConfiguration();
		}		
	}
	
	class DomainNewConfigurationMinimalListener implements ActionListener	
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().addNewMinimalConfiguration();
		}		
	}
	
	class DomainExtendListener implements ActionListener	
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().extendConfiguration();
		}		
	}
	
	class DomainNewConstraintListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().addNewConstraint();
		}		
	}
	
	class DomainCheckListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			SpecificationILPExport ilp = new SpecificationILPExport(mainFrame.getDomain().getSpecification());
			
			ilp.export();			

			ilp.setObjective(OptType.MIN);
			Result r = ilp.solve();
			ilp.printProblem();			
			mainFrame.getTextOutput().setText(
					mainFrame.getTextOutput().getText()+"\nMIN Result: "+r);
			
			ilp.setObjective(OptType.MAX);
			r = ilp.solve();
			mainFrame.getTextOutput().setText(
					mainFrame.getTextOutput().getText()+"\nMAX Result: "+r);
		}		
	}
	
	class ViewResetListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().resetView(); 
		}		
	}
	
	class ViewDetailsListener implements ActionListener
	{
		ViewDetails details;
		
		public ViewDetailsListener(ViewDetails vd)
		{
			details = vd;
		}
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
			Environment.VIEW_DETAILS = details;
		}		
	}
	
	
	
	class ViewZoomInListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().zoomIn();
		}		
	}
	
	class ViewZoomOutListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().zoomOut();
		}		
	}
	
	class ViewMoveLeftListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().moveLeft();
		}		
	}
	
	class ViewMoveRightListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().moveRight();
		}		
	}
	
	class ViewMoveUpListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().moveUp();
		}		
	}
	
	class ViewMoveDownListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDiagramPanel().moveDown();
		}		
	}
	
	class FileNewListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainFrame.getDomain().newDomain();
			mainFrame.getDomain().getHistory().clearUndoRedo();
		}		
	}

	class FileOpenListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
		    JFileChooser chooser = new JFileChooser();		    
		    CustomFileFilter filter = new CustomFileFilter(new String[]{Environment.DOMAIN_ENDING},"Clews Domains (*."+Environment.DOMAIN_ENDING+")");
		    chooser.setFileFilter(filter);
		    
		    int returnVal = chooser.showOpenDialog(mainFrame);
		    if(returnVal == JFileChooser.APPROVE_OPTION) 
		    {
		       String path = chooser.getSelectedFile().getAbsolutePath();
		       DomainXMLImport dxe = new DomainXMLImport();
		       if (!path.endsWith("."+Environment.DOMAIN_ENDING) || !(new File(path).exists()) ||
		    	   dxe.loadFromDisk(path) == false)
		    	   JOptionPane.showMessageDialog(mainFrame, path+" is not a valid Clews domain file","Error while reading file...",JOptionPane.ERROR_MESSAGE);
		       else
		       {
		    	   mainFrame.getDomain().getHistory().clearUndoRedo();
		    	   Domain d = mainFrame.getDomain();
	    		   d.newDomain(false);
	    		   d.setPath(path);
	    		   d.setConfigurations(dxe.getConfigurations());
	    		   d.setSpecification(dxe.getSpecification());	    
	    		   mainFrame.setMessageText("Domain loaded from "+path);
		       }
		    }			
		}		
	}
	
	class FileExportImageListener implements ActionListener
	{
		protected String ending;
		protected boolean screen;
		
		public FileExportImageListener(String end, boolean s)
		{
			ending = end;
			screen = s;
		}
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
		    JFileChooser chooser = new JFileChooser();		    
		    CustomFileFilter filter = new CustomFileFilter(new String[]{ending},ending.toUpperCase()+" (*."+ending+")");
		    chooser.setFileFilter(filter);
		    
		    int returnVal = chooser.showSaveDialog(mainFrame);
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
		    	String path = chooser.getSelectedFile().getAbsolutePath(); 
		    	if (!path.endsWith("."+ending))
		    		path = path + "."+ending;
		    	if (!mainFrame.getDiagramPanel().export(path,screen))
		    		JOptionPane.showMessageDialog(mainFrame, 
		    				"Exporting diagram to "+path+" caused an error",
		    				"Error while exporting file...",JOptionPane.ERROR_MESSAGE);
		    	else
		    		mainFrame.setMessageText("Image exported in "+path);
		    }
		}		
	}
	
	class FileSaveListener implements ActionListener
	{
		boolean forceDialog;
		public FileSaveListener(boolean fd)
		{
			forceDialog = fd;
		}
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
			String path = mainFrame.getDomain().getPath();
			if (forceDialog || path == null || path.contains(Environment.HISTORY_PATH))
			{
			    JFileChooser chooser = new JFileChooser();		    
			    CustomFileFilter filter = new CustomFileFilter(new String[]{Environment.DOMAIN_ENDING},"Clews Domains (*."+Environment.DOMAIN_ENDING+")");
			    chooser.setFileFilter(filter);
			    
			    int returnVal = chooser.showSaveDialog(mainFrame);
			    if(returnVal == JFileChooser.APPROVE_OPTION)
			    	save(chooser.getSelectedFile().getAbsolutePath());
			}
			else
				save(path);
			mainFrame.getDomain().getHistory().clearUndoRedo();
		}		
		
		public void save(String path)
		{
		   	if (!path.endsWith("."+Environment.DOMAIN_ENDING))
	    		path = path + "." + Environment.DOMAIN_ENDING;
		   	DomainXMLExport dxe = new DomainXMLExport(MainFrame.getInstance().getDomain());
		   	if (dxe.saveToDisk(path))
		   		mainFrame.setMessageText("Domain saved in "+path);
		   	else
		   		mainFrame.setMessageText("Error while saving domain");			
		}
	}

	class FileImportSpecifciation implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
		}		
	}
	
	class FileImportConfiguration implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
		}		
	}
	
	class FileExitListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			System.exit(0);
		}		
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (mainFrame != null)
			mainFrame.repaint();
	}

}
