package clews.gui.diagram;

import java.awt.Graphics2D;

import java.awt.Point;
import java.util.ArrayList;

import javax.swing.SwingUtilities;

import clews.MainFrame;
import clews.History.Operation;
import clews.gui.view.ConfigurationView;
import clews.gui.view.InstanceView;
import clews.gui.view.LinkView;
import clews.gui.view.PickableObject;

public class ConfigurationDiagram
extends Diagram<InstanceView, LinkView>
{
	protected ConfigurationView configurationView;
  
    public ConfigurationDiagram(ConfigurationView cv)
    {
    	super(MainFrame.getInstance().getDiagramPanel());
    	configurationView = cv;
		initToolTip();
    }
    
    public ConfigurationView getConfigurationView()
    {
    	return configurationView;
    }
    
    public String getName()
    {
    	return configurationView.getConfiguration().getName();
    }
    
    public void connect(PickableObject po1, PickableObject po2)
    {
    	MainFrame.getInstance().getDomain().getHistory().addUndo(null, Operation.ADD, "Link added");
    	LinkView lv = configurationView.addNewLinkView(
    		configurationView.getConfiguration().link(
    			((InstanceView)po1).getInstance(),((InstanceView)po2).getInstance()));
    	applyChanges();    	
    }
    
    @Override
    public void checkToolTip(PickableObject po, Point p)
	{	
    	if (po instanceof InstanceView)
    	{
    		InstanceView iv = (InstanceView)po;
	    	if (!iv.getInstance().isConsistent()){
	            if(!toolTip.isVisible()) {		                    
	                toolTipLabel.setText(iv.getInstance().reportConflicts());
	                toolTip.pack();
	                toolTip.setVisible(true);
	            }
	            hovering = true;
	            SwingUtilities.convertPointToScreen(p,this.panel);
	            toolTip.setLocation(p.x+5, p.y-toolTip.getHeight()-5);
			}
    	}
	}

	@Override
	public void draw(Graphics2D g) {
		if (configurationView != null)
			configurationView.draw(g);
		super.draw(g);
	}

	@Override
	public ArrayList<InstanceView> getPickableObjects() {
		return configurationView.getInstances();
	}

	@Override
	public ArrayList<LinkView> getSelectableObjects() {
		return configurationView.getLinks();
	}

	@Override
	public void applyChanges() {
		configurationView.applyChanges();
		MainFrame.getInstance().update();		

	}

	
}
