package clews.gui.diagram;

import java.awt.AlphaComposite;


import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;

import javax.imageio.ImageIO;
import javax.swing.JDialog;
import javax.swing.JPanel;

import net.sf.epsgraphics.ColorMode;
import net.sf.epsgraphics.EpsGraphics;

import clews.MainFrame;
import clews.env.Environment;
import clews.env.Utils;
import clews.gui.PanelDropTarget;
import clews.gui.dialogs.EditClassDialog;
import clews.gui.view.SpecificationView;

public class DiagramPanel 
extends JPanel 
implements MouseListener, MouseMotionListener, MouseWheelListener
{
	private static final long serialVersionUID = 2502690645254790583L;
	protected AnyDiagram diagram;
	protected RenderingHints renderHints;
	protected boolean controlDown;
	protected int lastx, lasty;
	protected Point selectionUpper, selectionLower;
	protected PanelDropTarget dt;
	protected MainFrame mainFrame;
	
//	protected Component cur = null;
	
	public DiagramPanel(MainFrame mf){
		mainFrame = mf;
		dt = new PanelDropTarget(this);
	    KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(
	    		new KeyEventDispatcher() 
	    		{ 
	    			public boolean dispatchKeyEvent( KeyEvent e ) 
	    			{
	    				if (e.isControlDown() != Environment.IS_CONTROL_DOWN)
	    				{
	    					Environment.IS_CONTROL_DOWN = e.isControlDown();
	    					controlDownChanged();
	    				}	    				
	    				if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_P)
	    				{
	    					//TODO
	    					export("d:\\fig.eps",true);
	    				}
	    				return false; // do not consume key event
	    			}
	    		}
	    );

		
		setBackground(Environment.DIAGRAM_BACKGROUND);

		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);
				
		renderHints  = new RenderingHints(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		renderHints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
	}
	
	public void changeSelectedDiagram(AnyDiagram newDiagram)
	{
		if (diagram != null)
		{
			removeMouseListener(diagram);
			removeMouseMotionListener(diagram);
		}
		diagram = newDiagram;
		if (diagram != null)
		{
			addMouseListener(diagram);
			addMouseMotionListener(diagram);
		}	
		this.updateUI();
	}
	
	public void mousePressed(MouseEvent e){
		
		if (!diagram.hasSelection() && !diagram.isOverObject())
		{
			diagram.clearDragList();
			if (e.getButton() == MouseEvent.BUTTON1)
			{
				selectionUpper = e.getPoint();
			}
			else
			{
	//			System.out.print("x="+e.getX()+" -> ");
				diagram.getInverseTransform().transform(e.getPoint(), e.getPoint());
	//			System.out.println("x="+e.getX());
				setBackground(Environment.DIAGRAM_SELECTED_BACKGROUND);
				lastx = (int)diagram.getTransform().getTranslateX() - e.getX();
			    lasty = (int)diagram.getTransform().getTranslateY() - e.getY();
			}
		}
	}
	
	public void mouseDragged(MouseEvent e){
		if (!diagram.hasSelection() && !diagram.isOverObject())
		{			
			if (selectionUpper != null)
			{
				selectionLower = e.getPoint();
			}
			else
			{
				diagram.getInverseTransform().transform(e.getPoint(), e.getPoint());
				double s = diagram.getTransform().getScaleX();
				diagram.getTransform().setToTranslation(lastx + e.getX(), lasty + e.getY());
				diagram.getTransform().scale(s, s);
			}
			repaint();
		}
	}
	
	public void mouseReleased(MouseEvent e){
		setBackground(Environment.DIAGRAM_BACKGROUND);
		if (selectionLower != null && selectionUpper != null) {
			Point2D sl = diagram.getInverseTransform().transform(new Point2D.Double(selectionLower.x,selectionLower.y), null);
			Point2D su = diagram.getInverseTransform().transform(new Point2D.Double(selectionUpper.x,selectionUpper.y), null);
			
			Rectangle r = new Rectangle(
					(int)Math.min(su.getX(),sl.getX()),
					(int)Math.min(su.getY(),sl.getY()),
					(int)Math.abs(sl.getX()-su.getX()),
					(int)Math.abs(su.getY()-sl.getY())
					);
			diagram.select(r);		
		}	
		selectionLower = selectionUpper = null;				
		repaint();
	}
	   
	public void mouseMoved(MouseEvent e){}	
	public void mouseClicked(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}                     
 	 
	
	protected void controlDownChanged()
	{
		diagram.clearSelection();
		repaint();
	}
	
	public void paintComponent(Graphics g){
	    super.paintComponent(g);
	    Graphics2D g2 = (Graphics2D)g;
    	g2.setRenderingHints(renderHints);
    	drawDiagram(g2);
	}	
	
	public boolean export(String filename, boolean screen)
	{
		try{			

			Dimension dim = diagram.setUpPrint();
			if (screen){
				if (Environment.CURRENT_DIALOG != null)
					dim = Environment.CURRENT_DIALOG.getSize();
				else
					dim = MainFrame.getInstance().getSize();
			}
			if (!Utils.getExtension(filename).equals("eps"))
			{
				BufferedImage bi = 
					new BufferedImage(dim.width, dim.height, BufferedImage.TYPE_INT_ARGB);
				Graphics2D g2 = bi.createGraphics();
				g2.setRenderingHints(renderHints);		
				
				g2.setColor(Color.WHITE);
				System.out.println(dim);
				g2.fillRect(0, 0, dim.width, dim.height);
				if (screen)
					MainFrame.getInstance().paint(g2);
				else			
					drawDiagram(g2,true);						
				File outputfile = new File(filename);
				outputfile.createNewFile();			
				ImageIO.write(bi, Utils.getExtension(filename), outputfile);
			}
			else
			{
				EpsGraphics g2 = new EpsGraphics("Clews Diagram Export",new FileOutputStream(new File(filename)),0,0,dim.width,dim.height,ColorMode.COLOR_RGB);
				g2.setRenderingHints(renderHints);		
				
				g2.setColor(Color.WHITE);
				g2.fillRect(0, 0, dim.width, dim.height);
				if (screen){
					if (Environment.CURRENT_DIALOG != null)
						Environment.CURRENT_DIALOG.paint(g2);
					else
						MainFrame.getInstance().paint(g2);					
				}
				else
					drawDiagram(g2,true);
				g2.close();
			}
		}catch (Exception ex)
		{
			ex.printStackTrace();
			return false;
		}
		return true;
	}

	public void drawDiagram(Graphics2D g2)
	{
		drawDiagram(g2,false);
	}
	
	public void drawDiagram(Graphics2D g2, boolean isPrinting)
	{
		if (diagram != null)
		{
			
			AffineTransform save = g2.getTransform();
			if (!isPrinting)
				g2.transform(diagram.getTransform());	
			else
				g2.transform(diagram.getPrintShift());
			
	    	diagram.draw(g2);    

	    	g2.setTransform(save);
		}
		if (!isPrinting)
			drawSelectionRect(g2);
	}
	
	public void drawSelectionRect(Graphics2D g2)
	{
		if (selectionLower != null && selectionUpper != null)
	    {
	    	g2.setColor(Color.BLUE);
	    	g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.DIAGRAM_SELECTION_ALPHA));
	    	
	    	Dimension dim = new Dimension(Math.abs(selectionLower.x-selectionUpper.x),
					Math.abs(selectionLower.y-selectionUpper.y));
	    	Rectangle rect;
	    	rect = new Rectangle(new Point(Math.min(selectionLower.x, selectionUpper.x),
	    					Math.min(selectionLower.y, selectionUpper.y)),dim);
	    	
	    	g2.fill(rect);
	    	g2.draw(rect);    	
	    	
	    	g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.DIAGRAM_SELECTION_ALPHA));
	    }	
	}

 	
	public void classDroped(clews.data.Class c, Point p)
	{
		mainFrame.getDomain().classDroped(c,p);
	}

	public AnyDiagram getDiagram()
	{
		return diagram;
	}		
	
	public void resetView()
	{
		if (diagram != null)
		{
			diagram.setTransform(AffineTransform.getScaleInstance(1.0, 1.0));
			diagram.getTransform().setToTranslation(0.0, 0.0);
		}
	}
	
	public void zoomIn()
	{		
		if (diagram != null)
			diagram.getTransform().scale(Environment.USER_ZOOM_IN,Environment.USER_ZOOM_IN);
	}
	
	public void zoomOut()
	{
		if (diagram != null)
			diagram.getTransform().scale(Environment.USER_ZOOM_OUT, Environment.USER_ZOOM_OUT);
	}
	
	public void moveLeft() 
	{ 
		moveView(-Environment.USER_TRANSLATION,0); 
	}
	
	public void moveRight() 
	{
		moveView(Environment.USER_TRANSLATION,0); 
	}
	
	public void moveUp()
	{ 
		moveView(0,-Environment.USER_TRANSLATION); 
	}
	
	public void moveDown()
	{
		moveView(0,Environment.USER_TRANSLATION); 
	}
	
	protected void moveView(double x, double y)
	{
		if (diagram != null)
			diagram.getTransform().translate(x, y);
	}
	
	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {
		if (e.getWheelRotation() < 0)
			zoomIn();
		else
			zoomOut();
		repaint();
	}
}

