package clews.gui.dialogs;

import java.awt.BorderLayout;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import clews.MainFrame;
import clews.data.Association;
import clews.data.AssociationAttribute;
import clews.env.Environment;

public class EditAssociationDialog
extends JDialog
{

	private static final long serialVersionUID = 327340088344680920L;
	protected Association assoc;
	protected JComboBox typeleft, typeright;
	protected JTextField minleft, maxleft, minright, maxright, name;
	protected JCheckBox showDetails;
	
	public EditAssociationDialog(Association a)
	{
		super(MainFrame.getInstance(),"Edit Association",Dialog.ModalityType.DOCUMENT_MODAL);
		Environment.CURRENT_DIALOG = this;
		Container parent = getContentPane();
		assoc = a;
		
		parent.setLayout(new BorderLayout());
		
		JPanel all = new JPanel();
		all.setLayout(new BorderLayout());
		
		JPanel left = new JPanel();
		JPanel right = new JPanel();
		left.setName("");
		
		left.setLayout(new FlowLayout());
		right.setLayout(new FlowLayout());
		
		left.setBorder(new TitledBorder("End "+assoc.getLeft().assclass.getName()));
		right.setBorder(new TitledBorder("End "+assoc.getRight().assclass.getName()));

		JPanel up = new JPanel();
		up.setLayout(new BorderLayout());
		up.add(new JLabel("Name: "),BorderLayout.WEST);
		up.add((name = new JTextField(assoc.getName())),BorderLayout.CENTER);
		up.add(showDetails=new JCheckBox("Show details",assoc.getView().getShowDetails()),BorderLayout.EAST);
		up.doLayout();

		
		all.add(up,BorderLayout.NORTH);
		all.add(left,BorderLayout.CENTER);
		all.add(right,BorderLayout.SOUTH);
		
		//parent.add(new JLabel("asdf"));
		
		typeleft = createCombo();
		typeright = createCombo();
		typeleft.setSelectedItem(assoc.getTypeLeft());
		typeright.setSelectedItem(assoc.getTypeRight());
		
		
		minleft = new JTextField(""+assoc.getMinLeft(),3);
		maxleft = new JTextField(assoc.getMaxLeft()==Environment.CARDINALITY_STAR?"*":""+assoc.getMaxLeft(),3);
		minright = new JTextField(""+assoc.getMinRight(),3);
		maxright = new JTextField(assoc.getMaxRight()==Environment.CARDINALITY_STAR?"*":""+assoc.getMaxRight(),3);
		
		left.add(new JLabel("Type: "));
		left.add(typeleft);
		left.add(new JLabel("Minimum: "));
		left.add(minleft);
		left.add(new JLabel("Maximum: "));
		left.add(maxleft);
		
		right.add(new JLabel("Type: "));
		right.add(typeright);
		right.add(new JLabel("Minimum: "));
		right.add(minright);
		right.add(new JLabel("Maximum: "));
		right.add(maxright);
		
		
		
		JPanel buttongrid = new JPanel();
		JButton delete = new JButton("Delete");
		JButton cancel = new JButton("Cancel");	
		JButton ok = new JButton("Ok");
		
		ok.addActionListener(new OkListener());
		cancel.addActionListener(new CancelListener());
		delete.addActionListener(new DeleteListener());
		
		buttongrid.setLayout(new BorderLayout());
		JPanel buttons = new JPanel();
		buttons.setLayout(new FlowLayout());
		buttons.add(delete);
		buttons.add(cancel);
		buttons.add(ok);
		buttongrid.add(buttons,BorderLayout.EAST);	
		
		parent.add(all,BorderLayout.CENTER);
		parent.add(buttongrid,BorderLayout.SOUTH);
		
		setResizable(false);
		setLocationRelativeTo(MainFrame.getInstance());
		pack();
	}
	
	protected JComboBox createCombo()
	{
		AssociationAttribute[] types = AssociationAttribute.values();
		return new JComboBox(types);
	}
	
	public void close()
	{
		Environment.CURRENT_DIALOG = null;
		setVisible(false);
	}
	
	class OkListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			
			if (name.getText() != null && name.getText().length() > 0)
				assoc.setName(name.getText());
			
			assoc.setTypes((AssociationAttribute)typeleft.getSelectedItem(),
					(AssociationAttribute)typeright.getSelectedItem());
			
			assoc.getView().setShowDetails(showDetails.isSelected());
			
			int mil=assoc.getMinLeft(), mir=assoc.getMinRight(),
			mal=assoc.getMaxLeft(),mar=assoc.getMaxRight();
			
			mil = getInteger(minleft,mil);
			mal = getInteger(maxleft,mal);
			mir = getInteger(minright,mir);
			mar = getInteger(maxright,mar);
			
			assoc.setCards(mil, mal, mir, mar);	
			close();
		}		
		
		public int getInteger(JTextField tf, int stdval)
		{
			int ret = stdval;
			if (tf.getText() != null && !tf.getText().equals(""))
			{
				if (tf.getText().equals("*"))
					ret = -1;
				else
				{
					try {
						ret = Integer.parseInt(tf.getText());
					}catch (Exception ex)
					{
					}
				}
			}
			return ret;
		}
	}

	class CancelListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			close();
		}		
	}
	
	class DeleteListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			assoc.getView().delete();
			close();
		}		
	}
}
