package clews.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import clews.MainFrame;
import clews.data.Association;
import clews.data.Constraint;
import clews.data.Specification;
import clews.data.Constraint.ConstraintPart;
import clews.env.Environment;

public class EditConstraintDialog
extends JDialog
{
	private static final long serialVersionUID = 3291489701988978561L;
	protected Constraint constraint;
	protected JTextField constraintName;
	protected JTextArea lhs, rhs;
	protected JCheckBox active;
	protected EditConstraintDialog instance;
	
	public EditConstraintDialog(Constraint c)
	{
		super(MainFrame.getInstance(),"Edit Constraint",Dialog.ModalityType.DOCUMENT_MODAL);
		Environment.CURRENT_DIALOG = this;
		instance = this;
		constraint = c;
		Container parent = getContentPane();	
		
		parent.setLayout(new BorderLayout());
		
		JPanel upp = new JPanel(new GridLayout(2,1));
		JPanel up = new JPanel(new BorderLayout());
		
		up.add(new JLabel("Name: "),BorderLayout.WEST);
		constraintName = new JTextField(c.getName());
		up.add(constraintName,BorderLayout.CENTER);
		up.add(active=new JCheckBox("Is constraint active"),BorderLayout.EAST);
		active.setSelected(constraint.isActive());
		
		JPanel labs = new JPanel(new BorderLayout());		
		labs.add(new JLabel("Left hand side"),BorderLayout.WEST);
		labs.add(new JLabel("Right hand side"),BorderLayout.EAST);
		
		JPanel p2 = new JPanel(new BorderLayout());		
		p2.add(new JScrollPane(lhs=new JTextArea(6, 22)),BorderLayout.WEST);
		p2.add(new JLabel(" = "),BorderLayout.CENTER);
		p2.add(new JScrollPane(rhs=new JTextArea(6, 22)),BorderLayout.EAST);
		
		for (ConstraintPart cp : constraint.getLhs())
			lhs.setText(lhs.getText()+cp.association.getName()+(cp.inv?Environment.ASSOCIATION_INVERSE:"")+"\n");
		
		for (ConstraintPart cp : constraint.getRhs())
			rhs.setText(rhs.getText()+cp.association.getName()+(cp.inv?Environment.ASSOCIATION_INVERSE:"")+"\n");
		
		
		JPanel bottom = new JPanel(new BorderLayout());
		JPanel p3 = new JPanel(new FlowLayout());
		JButton ok = new JButton("Ok");
		JButton cancel = new JButton("Cancel");
		JButton delete = new JButton("Delete");
		p3.add(delete);
		p3.add(cancel);
		p3.add(ok);
		bottom.add(p3,BorderLayout.EAST);
		
		JTextArea descs;
		JPanel desc = new JPanel(new BorderLayout());
		desc.add(new JLabel("Available associations"),BorderLayout.NORTH);
		desc.add(new JScrollPane(descs=new JTextArea()),BorderLayout.CENTER);
		desc.add(bottom,BorderLayout.SOUTH);
		
		descs.setEditable(false);
		String text = "";
		for (Association a : constraint.getSpecification().getAssociations())
			text = text + a.getName()+":   "+a.getLeft().assclass.getName() +" --- " + a.getRight().assclass.getName()+"\n";			
		descs.setRows(Math.min(10, constraint.getSpecification().getAssociations().size()+1));
		descs.setText(text);
		
		upp.add(up);
		upp.add(labs);
		parent.add(upp,BorderLayout.NORTH);
		parent.add(p2,BorderLayout.CENTER);
		parent.add(desc,BorderLayout.SOUTH);
		
		ok.addActionListener(new OkListener());
		cancel.addActionListener(new CancelListener());
		delete.addActionListener(new DeleteListener());
		
		
		setResizable(false);
		setLocationRelativeTo(MainFrame.getInstance());
		pack();
	}

	public void close()
	{
		Environment.CURRENT_DIALOG = null;
		setVisible(false);
	}
	
	class OkListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (constraintName != null && !constraintName.getText().equals(""))
				constraint.setName(constraintName.getText());
			constraint.setActive(active.isSelected());
			if (constraint.isActive())
			{
				constraint.clear();
				if (getAssociations(lhs,true) && getAssociations(rhs,false))
				{				
					if (checkParts(constraint.getLhs()) && checkParts(constraint.getRhs()))
					{
						if (constraint.computeResultingInterval())
							close();
						else
							JOptionPane.showMessageDialog(instance,constraint.getErrorMessage());
					}
				}
			}
			else
				close();
		}		
	}
	
	public boolean checkParts(ArrayList<ConstraintPart> acp)
	{
		for (int i=0;i<acp.size()-1;i++){
			if (!constraint.match(acp.get(i),acp.get(i+1)) ||
				!(acp.get(i).association.getTypeLeft() == acp.get(0).association.getTypeLeft())){
				JOptionPane.showMessageDialog(this,"Associations '"+acp.get(i).association.getName()+
						"' and '"+acp.get(i+1).association.getName()+"' do not match.");
				return false;
			}
		}
		return true;
	}
	
	public boolean getAssociations(JTextArea ta, boolean lhs)
	{
		String search;
		if (!ta.getText().equals("")){
			String[] strs = ta.getText().split("\n"); //not use: System.getProperty("line.separator")
			for (int i=0;i<strs.length;i++)
			{
				search = strs[i].replace(Environment.ASSOCIATION_INVERSE, "");
				if (search.equals(""))
					continue;
				Association assoc = constraint.getSpecification().getAssociationForName(search);
				if (assoc == null)
				{
					JOptionPane.showMessageDialog(this, 
							"Unknown association '"+search+"'");
					return false;
				}
				else
				{
					if (lhs)
						constraint.addLhsPart(assoc, strs[i].endsWith(Environment.ASSOCIATION_INVERSE));
					else
						constraint.addRhsPart(assoc, strs[i].endsWith(Environment.ASSOCIATION_INVERSE));
				}
			}
		}
		return true;
	}

	class CancelListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			close();
		}		
	}
	
	class DeleteListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent arg0) {
			constraint.getView().delete();
			close();
		}		
	}
}
