package clews.gui.view;

import java.awt.AlphaComposite;


import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.JOptionPane;

import clews.MainFrame;
import clews.History.Operation;
import clews.data.Association;
import clews.env.Environment;
import clews.env.Environment.ViewDetails;
import clews.gui.diagram.Diagram;
import clews.gui.dialogs.EditAssociationDialog;

public class AssociationView 
extends SelectableObject
{
	protected Association assoc;	
	protected Color color;
	protected SpecificationView specificationView;
	protected Rectangle2D outer,inner; 
	protected boolean showDetails;
	
	public AssociationView(SpecificationView sv, Association a)
	{
		super(true);
		specificationView = sv;
		assoc = a;
		color = Environment.ASSOCIATION_COLOR;
		showDetails = true;
		createPickingSelfJoin();
	}
	
	public void mark()
	{
		color = Environment.MOUSE_OVER_ASSOCIATION_COLOR;
	}
	
	public void unmark()
	{
		color = Environment.ASSOCIATION_COLOR;
	}
	
	public void setShowDetails(boolean show)
	{
		showDetails = show;
	}
	
	public boolean getShowDetails()
	{
		return showDetails;
	}
	
		
	public void draw(Graphics2D g)
	{
		AffineTransform save = g.getTransform();
//		g.transform(AffineTransform.getTranslateInstance(Diagram.printShiftX,Diagram.printShiftY));

		Composite ocomp = g.getComposite();
		if (Environment.IS_CONTROL_DOWN)
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.CLASS_SELECTION_ALPHA));
		
		Rectangle cv1=getLeftView().getBounds();
		Rectangle cv2=getRightView().getBounds();

		if (assoc.isConsistent() || !color.equals(Environment.ASSOCIATION_COLOR))
			g.setColor(color);
		else
			g.setColor(Environment.CLASS_COLOR_INCONSISTENT);
		
		
		if (cv1 != cv2)
		{			
			g.draw(getLine());

			drawEnd(g,cv1,cv2,assoc.getTypeLeft().name(),getCard(assoc.getMinLeft(),assoc.getMaxLeft(),assoc.getConstrainedMaxLeft()));
			drawEnd(g,cv2,cv1,assoc.getTypeRight().name(),getCard(assoc.getMinRight(),assoc.getMaxRight(),assoc.getConstrainedMaxRight()));
			if (showDetails && Environment.VIEW_DETAILS == ViewDetails.ALL)
				drawName(g,cv1,cv2);
		}
		else
		{
			g.draw(getSelfRect());
			
			drawEnd(g,cv1,new Rectangle(cv1.x,cv1.y-2*cv1.height,cv1.height,cv1.width),assoc.getTypeLeft().name(),getCard(assoc.getMinLeft(),assoc.getMaxLeft(),assoc.getConstrainedMaxLeft()));
			drawEnd(g,cv1,new Rectangle(cv1.x-2*cv1.width,cv1.y,cv1.height,cv1.width),assoc.getTypeRight().name(),getCard(assoc.getMinRight(),assoc.getMaxRight(),assoc.getConstrainedMaxRight()));
		}
		
		g.setComposite(ocomp);
		g.setTransform(save);
	}
	
	protected Rectangle getSelfRect()
	{
		Rectangle cv1=getLeftView().getBounds();
		Rectangle cv2=getRightView().getBounds();
		return new Rectangle(cv1.x-cv1.width, cv1.y-cv1.height, cv1.width*3/2, cv1.height*3/2);
	}
	
	public boolean contains(Point2D mp)
	{
		if (getLeftView() != getRightView())
			return (getLine().ptSegDist(mp) < Environment.ASSOCIATION_PICKING_SENSITVITY);
		else
		{
			createPickingSelfJoin();
			return (outer.contains(mp) && !inner.contains(mp));
		}
	}
	
	protected void createPickingSelfJoin()
	{
		Rectangle cv1=getLeftView().getBounds();

		double ps = Environment.ASSOCIATION_PICKING_SENSITVITY;
		outer = new Rectangle2D.Double(cv1.x-cv1.width-ps, cv1.y-cv1.height-ps, cv1.width*3/2+2*ps, cv1.height*3/2+2*ps);
		inner = new Rectangle2D.Double(cv1.x-cv1.width+ps, cv1.y-cv1.height+ps, cv1.width*3/2-ps, cv1.height*3/2-ps);
	}
	
	public Line2D getLine()
	{
		Rectangle cv1=getLeftView().getBounds();
		Rectangle cv2=getRightView().getBounds();

		return new Line2D.Double(cv1.getCenterX(), cv1.getCenterY(),cv2.getCenterX(), cv2.getCenterY());
	}
	
	protected String getCard(int min, int max, int cmax)
	{
		String ret = min+"..";
		if (max == -1)
			ret = ret+"*";
		else
			ret = ret+max;
		if (cmax != max)
			ret = ret + " / " + getCard(min,cmax,cmax);
		return ret;
	}
	
	@Override
	public boolean setOver(boolean isOver)
	{
		Color save = color;
		if (isOver)
			color = Environment.MOUSE_OVER_ASSOCIATION_COLOR;
		else
			color = Environment.ASSOCIATION_COLOR;
		return (save != color);
	}
	
	
	protected double getLineAngle(Rectangle cv1, Rectangle cv2)
	{
		double dx,dy,alpha;
		dx = cv2.getCenterX() - cv1.getCenterX();
		dy = cv2.getCenterY() - cv1.getCenterY();
		alpha = Math.acos((dx) / (Math.sqrt(dx*dx + dy*dy)));
		if (dy < 0)
			alpha = 2*Math.PI - alpha;
		//NEW
//		if (alpha >= Math.PI/2 && alpha < 3*Math.PI/2)
//			alpha += Math.PI;
		//NEW	
		return alpha;
	}
	
	protected double getLineLength()
	{
		Line2D l = getLine();
		return Math.sqrt(Math.pow(l.getX1()-l.getX2(), 2.0)+Math.pow(l.getY1()-l.getY2(), 2.0));
	}
	
	protected void drawName(Graphics2D g, Rectangle cv1, Rectangle cv2)
	{
		int arrowShift,mul;
		double shift,alpha=getLineAngle(cv1,cv2);
		Shape name = Environment.getSmallText(assoc.getName());
		
		AffineTransform save = g.getTransform();
		g.transform(AffineTransform.getTranslateInstance(cv1.getCenterX(), cv1.getCenterY()));
		
		shift = getLineLength()/2.0 - name.getBounds().width/2;
		g.rotate(alpha);
		if (alpha >= Math.PI/2 && alpha < 3*Math.PI/2)
		{			
			g.rotate(Math.PI);
			g.translate(-shift-name.getBounds().width, -name.getBounds().height-17);			
			mul = 1;
			arrowShift = 0;
		}
		else
		{
			g.translate(shift, -name.getBounds().height-17);
			arrowShift = name.getBounds().width;
			mul = -1;
		}

		g.fill(name);		
		
		int s=20;
		g.translate(mul*(-s-5), 0.0);
		Polygon p = new Polygon();
		p.addPoint(arrowShift+mul*(s-name.getBounds().height), name.getBounds().height/2);
		p.addPoint(arrowShift+mul*s, name.getBounds().height);
		p.addPoint(arrowShift+mul*s, 0);
		g.fill(p);
		
		g.setTransform(save);		
		
	}
	
	protected void drawEnd(Graphics2D g, Rectangle cv1, Rectangle cv2, String typestr, String cardstr)
	{
		AffineTransform save = g.getTransform();
		Color oldcolor = g.getColor();
		if (cardstr.contains("/"))
			g.setColor(Environment.ASSOCIATION_CONSTRAINT);
		
		int cardtrans;
		double shift,alpha=getLineAngle(cv1,cv2);

		g.transform(AffineTransform.getTranslateInstance(cv1.getCenterX(), cv1.getCenterY()));
		Shape card = Environment.getStandardText(cardstr), 
		type = Environment.getStandardText(typestr.replace("_", "-").toLowerCase());

		shift = (cv1.getWidth()/2+15) * scaleForAngle(alpha);
		if (alpha >= Math.PI/2 && alpha < 3*Math.PI/2)
		{			
			g.rotate(Math.PI);
			g.rotate(alpha);
			g.translate(-shift-type.getBounds().width, -type.getBounds().height-5);
			cardtrans = (type.getBounds().width - card.getBounds().width-10)/2;
		}
		else
		{
			g.rotate(alpha);
			g.translate(shift, -type.getBounds().height-5);
			cardtrans = (type.getBounds().width - card.getBounds().width-10)/2;
		}
		if (Environment.VIEW_DETAILS != ViewDetails.NO)
			g.fill(type);
		g.translate(cardtrans, type.getBounds().height+card.getBounds().height);
		g.fill(card);
		g.setTransform(save);
		g.setColor(oldcolor);
	}
	
	public double scaleForAngle(double angle)
	{
		double delta;		
		if (angle <= Math.PI/2 && angle >= 0)
			delta = angle - 0;
		else if (angle <= Math.PI && angle >= Math.PI/2)
			delta = angle - Math.PI/2;
		else if (angle <= 3*Math.PI/2 && angle >= Math.PI)
			delta = angle - Math.PI;
		else
			delta = angle - 3*Math.PI/2;
		
		if (delta > (Math.PI/4))
			delta = ((Math.PI/2)-delta);
			
		delta = delta / (Math.PI / 4);
		return 1 + (Math.sqrt(1.75)-1) * delta;
	}
	
	public ClassView getLeftView()
	{
		return assoc.getLeftClass().getView();
	}
	
	public ClassView getRightView()
	{
		return assoc.getRightClass().getView();
	}
	
	public Association getAssociationObject()
	{
		return assoc;
	}

	@Override
	public void clicked(Point2D p) {
		// TODO Auto-generated method stub
		
	}

	
	@Override
	public void release() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void rightClicked(Point2D p) {
		if (!Environment.IS_CONTROL_DOWN)
			if (JOptionPane.showConfirmDialog(MainFrame.getInstance().getDiagramPanel(), 
					"Sure you want delete "+assoc.getName(),
					"Delete...",JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				delete();
			}
	}

	@Override
	public void delete() {
		MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.DELETE, "Association deleted");
		specificationView.delete(this);		
	}
	
	@Override
	public void edit() {
		if (!Environment.IS_CONTROL_DOWN){
			try{
				MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.CHANGE, "Association changed");
				EditAssociationDialog ead = new EditAssociationDialog(assoc);
				ead.setVisible(true);
				specificationView.getDiagram().applyChanges();
			}
			catch (Exception ex)
			{
				// TODO: maybe java-bug occurs
			}
		}
	}

	@Override
	public Rectangle getPrintBounds() {
		Rectangle cv1=getLeftView().getBounds();
		Rectangle cv2=getRightView().getBounds();
		if (cv1 != cv2){
			Line2D l = getLine();
			int x1=(int)l.getX1(),x2=(int)l.getX2(),y1=(int)l.getY1(),y2=(int)l.getY2();
			return new Rectangle(Math.min(x1,x2), Math.min(y1, y2), Math.abs(x1-x2),Math.abs(y1-y2));
		}
		return getSelfRect();
	}

	@Override
	public boolean isOver() {
		 return color==Environment.MOUSE_OVER_ASSOCIATION_COLOR;
	}
}
