package clews.gui.view;

import java.awt.AlphaComposite;

import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;

import clews.MainFrame;
import clews.History.Operation;
import clews.env.Environment;
import clews.env.Environment.ViewDetails;
import clews.gui.diagram.Diagram;
import clews.gui.diagram.DiagramPanel;
import clews.gui.dialogs.EditClassDialog;
import clews.gui.dialogs.EditConstraintDialog;

public class ClassView 
extends PickableObject
{

	private static final long serialVersionUID = -4018680947156296745L;
	protected int lastx, lasty;
	protected Color color;
	protected SpecificationView specificationView;
	protected clews.data.Class classObj;
	protected Shape classtext;
	protected Rectangle classRect;
	
	public ClassView(SpecificationView sv, clews.data.Class c)
	{
		super(true);
		specificationView = sv;
		this.classObj = c;
		lastx = lasty = -1;
		color = Environment.CLASS_COLOR;
		isSelected = false;
		classRect = new Rectangle();
		setSizeFromName();		
		
		DiagramPanel dp = MainFrame.getInstance().getDiagramPanel();
		if (dp != null)
			setLocation(dp.getSize().width/2, dp.getSize().height/2);
	}
	
	public void setLocation(Point p)
	{
		classRect.setLocation(p);
	}
	
	@Override
	public Rectangle getBounds()
	{
		return classRect;
	}
	
	public void setSizeFromName()
	{
		classtext = null;
		if (classObj.getName() != null)
		{
			String orTxt="";
			if (classObj.getMinPartner() != 0 || classObj.getMaxParnter() != Environment.CARDINALITY_STAR){
				orTxt = "  {or["+classObj.getMinPartner()+".."+classObj.getMaxParnter()+"]}";
			}			
			classtext = Environment.getBoldText(classObj.getName()+orTxt);			
			int size = Math.max(classtext.getBounds().width, getMinSize())+20;
			classRect.setSize(size,size);
		}
	}
	
	public void setPosFromPosition(Point p)
	{
		setLocation(new Point(p.x-classRect.width/2,p.y-classRect.height/2));		
	}
		
	protected int getMinSize()
	{
		return Environment.getSmallText("MinObjects: ").getBounds().width+20;
	}
	
	public void draw(Graphics2D g)
	{
		AffineTransform old = g.getTransform();
//		g.transform(AffineTransform.getTranslateInstance(Diagram.printShiftX,Diagram.printShiftY));
		
		g.setColor(Environment.CLASS_INTERIOUR_COLOR);
		g.fill(classRect);
//		g.setStroke(Environment.DASHED);

		Composite ocomp = g.getComposite();
		if (Environment.IS_CONTROL_DOWN && !isSelected)
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.CLASS_SELECTION_ALPHA));

		if (classObj.isConsistent() || !color.equals(Environment.CLASS_COLOR))
			g.setColor(color);
		else
			g.setColor(Environment.CLASS_COLOR_INCONSISTENT);
		
		g.draw(classRect);
		
		if (classtext != null)
		{
			Shape last;
			AffineTransform save = g.getTransform();
			g.transform(AffineTransform.getTranslateInstance(classRect.x+5, classRect.y+5));
			g.fill(classtext);
			//System.out.println(classtext.getBounds().height);
			g.transform(AffineTransform.getTranslateInstance(0, 10+classtext.getBounds().height));
			g.fill(last=Environment.getItalicText("MinObjects: "+classObj.getMinObj()));
//			g.transform(AffineTransform.getTranslateInstance(0,last.getBounds().height+2));
//			g.fill(last=Environment.getItalicText("MaxObjects: "+classObj.getMinObj()));
			
			if (Environment.VIEW_DETAILS != ViewDetails.NO) {
				// new 
				if (classObj.getMaxObj() != Environment.CARDINALITY_STAR){
					g.transform(AffineTransform.getTranslateInstance(0, last.getBounds().height));
					g.fill(Environment.getItalicText("MaxObjects: "+classObj.getMaxObj()));
				}
				// new

				g.transform(AffineTransform.getTranslateInstance(0, last.getBounds().height));
				g.fill(Environment.getItalicText("Weight: "+classObj.getWeight()));
//				g.transform(AffineTransform.getTranslateInstance(0, last.getBounds().height+2));
//				g.fill(Environment.getItalicText("MaxChildren: "+classObj.getWeight()));
				
				g.transform(AffineTransform.getTranslateInstance(0,last.getBounds().height+2));
				String text = "MinModel: ";
				if (specificationView.getSpecification().isConsistent())
					text += classObj.getMinModelObj();
				else
					text += "---";
				g.fill(last=Environment.getItalicText(text));	
			}					
			g.setTransform(save);
		}	
		g.setComposite(ocomp);
		g.setTransform(old);
	}
	
	public void clicked(Point2D p)
	{		
		 lastx = classRect.x - (int)p.getX();
	     lasty = classRect.y - (int)p.getY();
	     color = Environment.MARKED_CLASS_COLOR;
	}
	
	public void mark()
	{
		color = Environment.MARKED_CLASS_COLOR;
	}
	
	public void unmark()
	{
		color = Environment.CLASS_COLOR;
	}
	
	@Override
	public boolean setOver(boolean isOver)
	{
		Color save = color;
		if (isOver)
			color = Environment.MOUSE_OVER_CLASS_COLOR;
		else
			color = Environment.CLASS_COLOR;
		return (save != color);
	}
	
	public void release()
	{
		color = Environment.CLASS_COLOR;
	}
	
	public void updateLocation(Point2D p)
	{		
		classRect.setLocation(lastx + (int)p.getX(), lasty + (int)p.getY());
//TODO		specificationView.setRepaint();
	}
	
	public Color getColor()
	{
		return color;
	}
	
	public clews.data.Class getClassObject()
	{
		return classObj;
	}

	@Override
	public boolean contains(Point2D p) {
		return classRect.contains(p);
	}


	@Override
	public void rightClicked(Point2D p) {
		if (!Environment.IS_CONTROL_DOWN)
			if (JOptionPane.showConfirmDialog(MainFrame.getInstance().getDiagramPanel(), 
					"Sure you want delete "+classObj.getName(),
					"Delete...",JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				delete();
			}		
	}

	@Override
	public void delete() {
		MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.DELETE, "Class deleted");
		specificationView.delete(this);
	}
	
	@Override
	public void edit() {
		if (!Environment.IS_CONTROL_DOWN){
			try{
				MainFrame.getInstance().getDomain().getHistory().addUndo(null, Operation.CHANGE, "Class changed");
				EditClassDialog ecd = new EditClassDialog(classObj);
				Object save = this.clone();
				ecd.setVisible(true);
				setSizeFromName();
				specificationView.getDiagram().applyChanges();
			}catch (Exception ex)
			{
				// TODO: maybe java-bug occurs
				ex.printStackTrace();
			}
		}
	}

	@Override
	public Rectangle getPrintBounds() {
		return getBounds();
	}

	@Override
	public boolean isOver() {
		return color == Environment.MARKED_CLASS_COLOR;
	}
}
