package clews.gui.view;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;

import clews.MainFrame;
import clews.History.Operation;
import clews.data.Configuration;
import clews.data.Instance;
import clews.data.Link;
import clews.gui.diagram.ConfigurationDiagram;

public class ConfigurationView 
extends ViewObject
{
	protected ConfigurationDiagram diagram;
	protected Configuration config;
	protected ArrayList<LinkView> links;
	protected ArrayList<InstanceView> instances;
	
	public ConfigurationView(Configuration c)
	{
		config = c;
		diagram = new ConfigurationDiagram(this);
		links = new ArrayList<LinkView>();
		instances = new ArrayList<InstanceView>();
		c.setPath(null);
	}
		
	public ConfigurationDiagram getDiagram()
	{
		return diagram;
	}
	
	public void delete(InstanceView iv)
	{		
		config.delete(iv.getInstance());
		instances.remove(iv);
		ArrayList<Link> toDel = new ArrayList<Link>();
		for (Number n : iv.getInstance().getLinks().keySet())
			for (Link l : iv.getInstance().getLinks().get(n))
				toDel.add(l);
		for (Link l : toDel)
			delete(l.getView());
		diagram.applyChanges();
	}
	
	public void delete(LinkView lv)
	{
		config.delete(lv.getLink());
		links.remove(lv);
		diagram.applyChanges();
	}
	
	public void clear()
	{
		links.clear();
		instances.clear();
	}
	
	public Configuration getConfiguration()
	{
		return config;
	}
	
	public Instance addDropInstance(clews.data.Class c, Point p)
	{
		Instance inst = config.addInstance(c);
		InstanceView iv = new InstanceView(inst, p);
		inst.setView(iv);
		instances.add(iv);
		return inst;
	}
	
	public ArrayList<InstanceView> getInstances()
	{
		return instances;
	}

	public LinkView addNewLinkView(Link l)
	{
		LinkView lv = new LinkView(l);
		l.setView(lv);
		links.add(lv);
		return lv;
	}
	
	public ArrayList<LinkView> getLinks()
	{
		return links;
	}
	
	public void applyChanges()
	{
		config.applyChanges();
	}
	
	public void draw(Graphics2D g)
	{
		ArrayList<ViewObject> todraw = new ArrayList<ViewObject>();
		for (LinkView lv : links)
		{
			if (lv.isOver())
				lv.draw(g);
			else
				todraw.add(lv);
		}
		for (ViewObject vo : todraw)
			vo.draw(g);
		todraw = new ArrayList<ViewObject>();
		for (InstanceView iv : instances)
		{
			if (iv.isSelected() || iv.isOver())
				iv.draw(g);
			else
				todraw.add(iv);
		}
		
		for (ViewObject vo : todraw)
			vo.draw(g);
	}

	@Override
	public Rectangle getPrintBounds() {
		return null;
	}	
}
