package clews.gui.view;

import java.awt.AlphaComposite;

import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;

import clews.MainFrame;
import clews.History.Operation;
import clews.data.Instance;
import clews.env.Environment;
import clews.gui.dialogs.EditInstanceDialog;

public class InstanceView 
extends PickableObject
{

	protected int lastx, lasty;
	protected Color color;
	protected Instance instance;
	protected Point pos;
	protected int radius;
	protected Shape insttext;
	
	public InstanceView(Instance in, Point p)
	{
		super(true);
		instance = in;
		setSizeFromName();
		pos = new Point(p.x-(int)getRadius(),p.y-(int)getRadius());
		isSelected = false;
		unmark();
	}
	
	public Instance getInstance()
	{
		return instance;
	}
	
	public double getRadius()
	{
		return (double)radius / 2.0;
	}

	public void setSizeFromName()
	{
		insttext = null;
		setPositionChanged();
		if (instance.getName() != null)
		{
			insttext = Environment.getStandardText(instance.getName());
			radius = insttext.getBounds().width+12;			
		}
	}
	
	public void draw(Graphics2D g)
	{
		g.setColor(Color.WHITE);
		g.fillOval(pos.x, pos.y, radius, radius);
		Composite ocomp = g.getComposite();
		if (Environment.IS_CONTROL_DOWN && !isSelected)
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.CLASS_SELECTION_ALPHA));
		
		Color col = color;
		if (!instance.isConsistent() && col.equals(Environment.CLASS_COLOR))
			col = Environment.CLASS_COLOR_INCONSISTENT;

		
		g.setColor(col);
		g.drawOval(pos.x, pos.y, radius, radius);
		
		
		AffineTransform save = g.getTransform();
		g.transform(AffineTransform.getTranslateInstance(radius/2+pos.x-insttext.getBounds().width/2, radius/2+pos.y-insttext.getBounds().height/2));
				
		g.setColor(col);
		g.fill(insttext);
		
		positionChanged = false;
		
		g.setComposite(ocomp);
		g.setTransform(save);
	}
	
	public Point2D getCenter()
	{
		return new Point2D.Double(pos.x+radius/2,pos.y+radius/2);
	}
	
	@Override
	public boolean contains(Point2D p)
	{
		Point2D m = getCenter(); 
		double x=m.getX()-p.getX(),y=m.getY()-p.getY();
		return Math.sqrt(x*x + y*y) <= radius/2;
	}
	
	@Override
	public Rectangle getBounds()
	{
		return new Rectangle(pos, new Dimension(radius,radius));
	}
	
	public void clicked(Point2D p)
	{		
		 lastx = pos.x - (int)p.getX();
	     lasty = pos.y - (int)p.getY();
	     color = Environment.MARKED_CLASS_COLOR;
	}
	
	public void mark()
	{
		color = Environment.MARKED_CLASS_COLOR;
	}
	
	public void unmark()
	{
		color = Environment.CLASS_COLOR;
	}
	
	public boolean setOver(boolean isOver)
	{
		Color save = color;
		if (isOver)
			color = Environment.MOUSE_OVER_CLASS_COLOR;
		else
			color = Environment.CLASS_COLOR;
		return (save != color);
	}
	
	public void release()
	{
		color = Environment.CLASS_COLOR;
	}
	
	public void setLocation(Point p)
	{
		
	}
	
	public void updateLocation(Point2D p)
	{		
		pos = new Point(lastx + (int)p.getX(), lasty + (int)p.getY());
		positionChanged = true;
//TODO		instance.getConfiguration().getDiagram().getView().setRepaint();
	}
	
	public Color getColor()
	{
		return color;
	}


	@Override
	public void rightClicked(Point2D p) {
		if (!Environment.IS_CONTROL_DOWN)
			if (JOptionPane.showConfirmDialog(MainFrame.getInstance(), 
					"Sure you want delete "+instance.getName(),
					"Delete...",JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				delete();
			}
	}
	
	@Override
	public void delete() {
		MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.DELETE, "Instance deleted");
		instance.getConfiguration().getView().delete(this);
	}

	@Override
	public void edit() {
		if (!Environment.IS_CONTROL_DOWN){		
			try{
				MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.CHANGE, "Instance edited");
				EditInstanceDialog eid = new EditInstanceDialog(instance);
				eid.setVisible(true);
				MainFrame.getInstance().repaint();
			}catch (Exception ex)
			{
				// TODO: maybe java-bug occurs
			}
		}
	}

	@Override
	public Rectangle getPrintBounds() {
		return getBounds();
	}

	@Override
	public boolean isOver() {
		return color != Environment.MARKED_CLASS_COLOR;
	}
}
