package clews.gui.view;

import java.awt.AlphaComposite;

import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;

import clews.MainFrame;
import clews.History.Operation;
import clews.data.Link;
import clews.env.Environment;
import clews.gui.dialogs.EditInstanceDialog;
import clews.gui.dialogs.EditLinkDialog;

public class LinkView 
extends SelectableObject
{
	protected Link link;
	protected Color color;
	protected int cntleft, cntright;
	protected Line2D.Double line;
	
	public LinkView(Link l)
	{
		link = l;
		cntleft = l.getLeft().getNumberOfLinks(l.getRight());
		cntright = l.getRight().getNumberOfLinks(l.getLeft());
		color = Environment.ASSOCIATION_COLOR;
		isEditable = true;
	}
	
	public Link getLink()
	{
		return link;
	}
	
	public void decreaseLeft()
	{
		cntleft--;
	}
	
	public void decreaseRight()
	{
		cntright--;	
	}
	
	public int getLeftCnt()
	{
		return cntleft;
	}
	
	public int getRightCnt()
	{
		return cntright;
	}
	
	public void mark()
	{
		color = Environment.MOUSE_OVER_ASSOCIATION_COLOR;
	}
	
	public void unmark()
	{
		color = Environment.ASSOCIATION_COLOR;
	}
	
	public boolean contains(Point2D mp)
	{
		return (getLine().ptSegDist(mp) < Environment.ASSOCIATION_PICKING_SENSITVITY);
	}
	
	@Override
	public boolean setOver(boolean isOver)
	{
		Color save = color;
		if (isOver)
			color = Environment.MOUSE_OVER_ASSOCIATION_COLOR;
		else
			color = Environment.ASSOCIATION_COLOR;
		return (save != color);
	}

	public Line2D getLine()
	{
		if (line == null || link.getLeft().getView().hasPositionChanged() || link.getRight().getView().hasPositionChanged())
		{
			double r1 = link.getLeft().getView().getRadius();
			double r2 = link.getRight().getView().getRadius();
		
			InstanceView iv1 = link.getLeft().getView();
			InstanceView iv2 = link.getRight().getView();
			
			Point2D c1 = iv1.getCenter();
			Point2D c2 = iv2.getCenter();
			
			Point2D dir = new Point2D.Double(-(c2.getY()-c1.getY()),c2.getX()-c1.getX());				
			double dist = dir.distance(0, 0);
	
			Point2D v1 = new Point2D.Double(dir.getX()/dist * (double)r1 , dir.getY()/dist * (double)r1 );
			Point2D v2 = new Point2D.Double(dir.getX()/dist * (double)r2 , dir.getY()/dist * (double)r2 );
								
			return (line=new Line2D.Double(
			(c1.getX()+v1.getX()-(2.0*v1.getX()/(link.getLeft().getNumberOfLinks(link.getRight())+1)*cntleft)),
			(c1.getY()+v1.getY()-(2.0*v1.getY()/(link.getLeft().getNumberOfLinks(link.getRight())+1)*cntleft)),
			(c2.getX()+v2.getX()-(2.0*v2.getX()/(link.getRight().getNumberOfLinks(link.getLeft())+1)*cntright)),
			(c2.getY()+v2.getY()-(2.0*v2.getY()/(link.getRight().getNumberOfLinks(link.getLeft())+1)*cntright))));
		}
		return line;
	}
	
	protected double getLineAngle(InstanceView cv1, InstanceView cv2)
	{
		double dx,dy,alpha;
		dx = cv2.getCenter().getX() - cv1.getCenter().getX();
		dy = cv2.getCenter().getY() - cv1.getCenter().getY();
		alpha = Math.acos((dx) / (Math.sqrt(dx*dx + dy*dy)));
		if (dy < 0)
			alpha = 2*Math.PI - alpha;
		//NEW
//		if (alpha >= Math.PI/2 && alpha < 3*Math.PI/2)
//			alpha += Math.PI;
		//NEW	
		return alpha;
	}
	
	protected double getLineLength()
	{
		Line2D l = getLine();
		return Math.sqrt(Math.pow(l.getX1()-l.getX2(), 2.0)+Math.pow(l.getY1()-l.getY2(), 2.0));
	}

	protected void drawName(Graphics2D g)
	{
		int arrowShift,mul;
		InstanceView iv1 = link.getLeft().getView(), iv2 = link.getRight().getView();
		
		double shift,alpha=getLineAngle(iv1,iv2);
		Shape name = Environment.getStandardText(link.getDescription());
		
		AffineTransform save = g.getTransform();
		g.transform(AffineTransform.getTranslateInstance(iv1.getCenter().getX(),iv1.getCenter().getY()+15));
		
		shift = getLineLength()/2.0 - name.getBounds().width/2;
		g.rotate(alpha);
		if (alpha >= Math.PI/2 && alpha < 3*Math.PI/2)
		{			
			g.rotate(Math.PI);
			g.translate(-shift-name.getBounds().width, -name.getBounds().height-17);			
			mul = 1;
			arrowShift = 0;
		}
		else
		{
			g.translate(shift, -name.getBounds().height-17);
			arrowShift = name.getBounds().width;
			mul = -1;
		}

		g.fill(name);		
		
/*		int s=20;
		g.translate(mul*(-s-5), 0.0);
		Polygon p = new Polygon();
		p.addPoint(arrowShift+mul*(s-name.getBounds().height), name.getBounds().height/2);
		p.addPoint(arrowShift+mul*s, name.getBounds().height);
		p.addPoint(arrowShift+mul*s, 0);
		g.fill(p);
*/		
		g.setTransform(save);		
		
	}

	
	
	public void draw(Graphics2D g)
	{
		Composite ocomp = g.getComposite();
		if (Environment.IS_CONTROL_DOWN)
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,Environment.CLASS_SELECTION_ALPHA));

		g.setColor(color);		
		g.draw(getLine());			
		if (link.getDescription() != null && link.getDescription().length() > 0)
			drawName(g);
		
		g.setComposite(ocomp);
	}


	@Override
	public void clicked(Point2D p) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void release() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void delete() {
		MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.DELETE, 
		"Link deleted");
		link.getConfiguration().getView().delete(this);
	}
	
	@Override
	public void rightClicked(Point2D p) {
		if (!Environment.IS_CONTROL_DOWN)
			if (JOptionPane.showConfirmDialog(MainFrame.getInstance(), 
					"Sure you want delete this link between "+link.getLeft().getName()+" and "+link.getRight().getName(),
					"Delete...",JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				delete();
			}		
	}
	
	@Override
	public void edit() {
		if (!Environment.IS_CONTROL_DOWN){		
			try{
				MainFrame.getInstance().getDomain().getHistory().addUndo(this, Operation.CHANGE, "Instance edited");
				EditLinkDialog eid = new EditLinkDialog(link);
				eid.setVisible(true);
				MainFrame.getInstance().repaint();
			}catch (Exception ex)
			{
				// TODO: maybe java-bug occurs
			}
		}
	}
	@Override
	public Rectangle getPrintBounds() {
		Line2D l = getLine();
		int x1=(int)l.getX1(),x2=(int)l.getX2(),y1=(int)l.getY1(),y2=(int)l.getY2();
		return new Rectangle(Math.min(x1,x2), Math.min(y1, y2), Math.abs(x1-x2),Math.abs(y1-y2));
	}

	@Override
	public boolean isOver() {
		return color==Environment.ASSOCIATION_COLOR;
	}
}
