package clews.gui.view;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.ArrayList;


import clews.data.Association;
import clews.data.Specification;
import clews.gui.diagram.SpecificationDiagram;

public class SpecificationView
extends ViewObject
{	
	protected SpecificationDiagram diagram;
	protected ArrayList<ClassView> classes;
	protected ArrayList<AssociationView> assocs;
	protected Specification specification;
	
	public SpecificationView(Specification s)
	{
		specification = s;
		diagram = new SpecificationDiagram(this);
		classes = new ArrayList<ClassView>();
		assocs = new ArrayList<AssociationView>();
		specification.setPath(null);
	}	
	
	public SpecificationDiagram getDiagram()
	{
		return diagram;
	}
	
	public ArrayList<ClassView> getClasses()
	{
		return classes;
	}
	
	public ArrayList<AssociationView> getAssociations()
	{
		return assocs;
	}	
	
		
	public void delete(ClassView cv)
	{
		ArrayList<Association> atodel = specification.delete(cv.getClassObject());
		for (Association a : atodel)
			assocs.remove(a.getView());
		classes.remove(cv);
		diagram.applyChanges();
	}
	
	public void delete(AssociationView av)
	{
		specification.delete(av.getAssociationObject());
		assocs.remove(av);
		diagram.applyChanges();
	}
	
	public void draw(Graphics2D g)
	{
		ArrayList<ViewObject> todraw = new ArrayList<ViewObject>();
		g.setStroke(new BasicStroke(1.0f));
		for (AssociationView av : assocs)
		{
			if (av.isOver())
				todraw.add(av);
			else
				av.draw(g);
		}
		for (ViewObject vo : todraw)
			vo.draw(g);
		todraw = new ArrayList<ViewObject>();
		for (ClassView cv : classes)
		{
			if (cv.isOver())
				todraw.add(cv);
			else
				cv.draw(g);			
		}
		for (ViewObject vo : todraw)
			vo.draw(g);
	}
	
	public void applyChanges()
	{
		specification.applyChanges();
	}
	
	public ClassView addNewClassView(clews.data.Class c)
	{
		ClassView cv = null;
		if (c != null)
		{
			cv = new ClassView(this,c);
			c.setView(cv);
			classes.add(cv);		
		}
		return cv;
	}
	
	public AssociationView addNewAssociationView(Association a)
	{
		AssociationView av = null;
		if (a != null)
		{
			av = new AssociationView(this, a);
			a.setView(av);
			assocs.add(av);
		}
		return av;

	}
	
	public Specification getSpecification()
	{
		return specification;
	}

	@Override
	public Rectangle getPrintBounds() {
		return null;
	}
}
